#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import argparse
import copy
import datetime
import uuid

from magnumclient.tests.osc.unit import osc_fakes
from magnumclient.tests.osc.unit import osc_utils


class FakeBaseModel(object):
    def __repr__(self):
        return "<" + self.__class__.model_name + "%s>" % self._info


class FakeBaseModelManager(object):
    def list(self, limit=None, marker=None, sort_key=None,
             sort_dir=None, detail=False):
        pass

    def get(self, id):
        pass

    def create(self, **kwargs):
        pass

    def delete(self, id):
        pass

    def update(self, id, patch):
        pass

    def rotate_ca(self, **kwargs):
        pass


class FakeStatsModelManager(object):
    def list(self, **kwargs):
        pass


class FakeQuotasModelManager(object):
    def get(self, id, resource):
        pass

    def create(self, **kwargs):
        pass

    def delete(self, id):
        pass


class FakeNodeGroupManager(object):
    def list(self, cluster_id, limit=None, marker=None, sort_key=None,
             sort_dir=None, detail=False):
        pass

    def get(self, cluster_id, id):
        pass

    def create(self, cluster_id, **kwargs):
        pass

    def delete(self, cluster_id, id):
        pass

    def update(self, cluster_id, id, patch):
        pass


class FakeCertificatesModelManager(FakeBaseModelManager):
    def get(self, cluster_uuid):
        pass


class MagnumFakeContainerInfra(object):
    def __init__(self):
        self.cluster_templates = FakeBaseModelManager()
        self.clusters = FakeBaseModelManager()
        self.mservices = FakeBaseModelManager()
        self.certificates = FakeCertificatesModelManager()
        self.stats = FakeStatsModelManager()
        self.quotas = FakeQuotasModelManager()
        self.nodegroups = FakeNodeGroupManager()


class MagnumFakeClientManager(osc_fakes.FakeClientManager):
    def __init__(self):
        super(MagnumFakeClientManager, self).__init__()
        self.container_infra = MagnumFakeContainerInfra()


class MagnumParseException(Exception):
    """The base exception class for all exceptions this library raises."""

    def __init__(self, message=None, details=None):
        self.message = message or "Argument parse exception"
        self.details = details or None

    def __str__(self):
        return self.message


class TestMagnumClientOSCV1(osc_utils.TestCase):

    def setUp(self):
        super(TestMagnumClientOSCV1, self).setUp()
        self.fake_stdout = osc_fakes.FakeStdout()
        self.fake_log = osc_fakes.FakeLog()
        self.app = osc_fakes.FakeApp(self.fake_stdout, self.fake_log)
        self.namespace = argparse.Namespace()
        self.app.client_manager = MagnumFakeClientManager()

    def check_parser(self, cmd, args, verify_args):
        cmd_parser = cmd.get_parser('check_parser')
        try:
            parsed_args = cmd_parser.parse_args(args)
        except SystemExit:
            raise MagnumParseException()
        for av in verify_args:
            attr, value = av
            if attr:
                self.assertIn(attr, parsed_args)
                self.assertEqual(value, getattr(parsed_args, attr))
        return parsed_args


class FakeClusterTemplate(object):
    """Fake one or more ClusterTemplate."""

    @staticmethod
    def create_one_cluster_template(attrs=None):
        """Create a fake ClusterTemplate.

        :param Dictionary attrs:
            A dictionary with all attributes
        :return:
            A FakeResource object, with flavor_id, image_id, and so on
        """

        attrs = attrs or {}

        # set default attributes.
        ct_info = {
            'links': [],
            'insecure_registry': None,
            'labels': None,
            'updated_at': None,
            'floating_ip_enabled': True,
            'fixed_subnet': None,
            'master_flavor_id': None,
            'uuid': uuid.uuid4().hex,
            'no_proxy': None,
            'https_proxy': None,
            'tls_disabled': False,
            'keypair_id': None,
            'public': False,
            'http_proxy': None,
            'docker_volume_size': None,
            'server_type': 'vm',
            'external_network_id': 'public',
            'cluster_distro': 'fedora-atomic',
            'image_id': 'fedora-atomic-latest',
            'volume_driver': None,
            'registry_enabled': False,
            'docker_storage_driver': 'devicemapper',
            'apiserver_port': None,
            'name': 'fake-ct-' + uuid.uuid4().hex,
            'created_at': datetime.datetime.now(),
            'network_driver': 'flannel',
            'fixed_network': None,
            'coe': 'kubernetes',
            'flavor_id': 'm1.medium',
            'master_lb_enabled': False,
            'dns_nameserver': '8.8.8.8',
            'project_id': uuid.uuid4().hex,
            'hidden': False,
            'tags': "",
        }

        # Overwrite default attributes.
        ct_info.update(attrs)

        ct = osc_fakes.FakeResource(info=copy.deepcopy(ct_info), loaded=True)
        return ct

    @staticmethod
    def create_cluster_templates(attrs=None, count=2):
        """Create multiple fake cluster templates.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param int count:
            The number of cluster templates to fake
        :return:
            A list of FakeResource objects faking the cluster templates
        """
        cts = []
        for i in range(0, count):
            cts.append(FakeClusterTemplate.create_one_cluster_template(attrs))

        return cts


class FakeCluster(object):
    """Fake one or more Cluster."""

    @staticmethod
    def create_one_cluster(attrs=None):
        """Create a fake Cluster.

        :param Dictionary attrs:
            A dictionary with all attributes
        :return:
            A FakeResource object, with flavor_id, image_id, and so on
        """

        attrs = attrs or {}

        # set default attributes.
        cluster_info = {
            'status': 'CREATE_IN_PROGRESS',
            'health_status': 'HEALTHY',
            'cluster_template_id': 'fake-ct',
            'node_addresses': [],
            'uuid': '3a369884-b6ba-484f-a206-919b4b718aff',
            'stack_id': 'c4554582-77bd-4734-8f1a-72c3c40e5fb4',
            'status_reason': None,
            'labels': {},
            'labels_overridden': {},
            'labels_added': {},
            'labels_skipped': {},
            'fixed_network': 'fixed-network',
            'fixed_subnet': 'fixed-subnet',
            'floating_ip_enabled': True,
            'created_at': '2017-03-16T18:40:39+00:00',
            'updated_at': '2017-03-16T18:40:45+00:00',
            'coe_version': None,
            'faults': None,
            'keypair': 'fakekey',
            'api_address': None,
            'master_addresses': [],
            'create_timeout': 60,
            'node_count': 1,
            'discovery_url': 'https://fake.cluster',
            'docker_volume_size': 1,
            'master_count': 1,
            'container_version': None,
            'name': 'fake-cluster',
            'master_flavor_id': None,
            'flavor_id': 'm1.medium',
            'project_id': None,
            'health_status_reason': {'api': 'ok'},
            'master_lb_enabled': False,
        }

        # Overwrite default attributes.
        cluster_info.update(attrs)

        cluster = osc_fakes.FakeResource(info=copy.deepcopy(cluster_info),
                                         loaded=True)
        return cluster


class FakeCert(object):
    def __init__(self, pem):
        self.pem = pem


class FakeQuota(object):
    """Fake one or more Quota"""

    @staticmethod
    def create_one_quota(attrs=None):
        """Create a fake quota

        :param Dictionary attrs:
            A dictionary with all attributes
        :return:
            A FakeResource object, with project_id, resource and so on
        """

        attrs = attrs or {}

        quota_info = {
            'resource': 'Cluster',
            'created_at': '2017-09-15T05:40:34+00:00',
            'updated_at': '2017-09-15T05:40:34+00:00',
            'hard_limit': 1,
            'project_id': 'be24b6fba2ed4476b2bd01fa8f0ba74e',
            'id': 10,
            'name': 'fake-quota',
        }

        quota_info.update(attrs)
        quota = osc_fakes.FakeResource(info=copy.deepcopy(quota_info),
                                       loaded=True)
        return quota


class FakeNodeGroup(object):
    """Fake one or more NodeGroup."""

    @staticmethod
    def create_one_nodegroup(attrs=None):
        """Create a fake NodeGroup.

        :param Dictionary attrs:
            A dictionary with all attributes
        :return:
            A FakeResource object, with flavor_id, image_id, and so on
        """

        attrs = attrs or {}

        # set default attributes.
        nodegroup_info = {
            'created_at': '2017-03-16T18:40:39+00:00',
            'updated_at': '2017-03-16T18:40:45+00:00',
            'uuid': '3a369884-b6ba-484f-a206-919b4b718aff',
            'cluster_id': 'fake-cluster',
            'docker_volume_size': None,
            'node_addresses': [],
            'labels': {},
            'labels_overridden': {},
            'labels_added': {},
            'labels_skipped': {},
            'node_count': 1,
            'name': 'fake-nodegroup',
            'flavor_id': 'm1.medium',
            'image_id': 'fedora-latest',
            'project_id': None,
            'role': 'worker',
            'max_node_count': 10,
            'min_node_count': 1,
            'is_default': False,
            'stack_id': '3a369884-b6ba-484f-fake-stackb718aff',
            'status': 'CREATE_COMPLETE',
            'status_reason': 'None',
            'master_lb_enabled': False,
        }

        # Overwrite default attributes.
        nodegroup_info.update(attrs)

        nodegroup = osc_fakes.FakeResource(info=copy.deepcopy(nodegroup_info),
                                           loaded=True)
        return nodegroup



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneauth1.exceptions import catalog
from keystoneauth1 import session as ksa_session
from openstack import config as occ
from oslo_utils import importutils

from magnumclient.common import httpclient
from magnumclient.v1 import certificates
from magnumclient.v1 import cluster_templates
from magnumclient.v1 import clusters
from magnumclient.v1 import credentials
from magnumclient.v1 import mservices
from magnumclient.v1 import nodegroups
from magnumclient.v1 import quotas
from magnumclient.v1 import stats

profiler = importutils.try_import("osprofiler.profiler")


DEFAULT_SERVICE_TYPE = 'container-infra'
LEGACY_DEFAULT_SERVICE_TYPE = 'container'


def _load_session(cloud=None, insecure=False, timeout=None, **kwargs):
    cloud_config = occ.OpenStackConfig()
    cloud_config = cloud_config.get_one(
        cloud=cloud,
        verify=not insecure,
        **kwargs)
    verify, cert = cloud_config.get_requests_verify_args()

    auth = cloud_config.get_auth()
    session = ksa_session.Session(
        auth=auth, verify=verify, cert=cert,
        timeout=timeout)

    return session


def _load_service_type(session,
                       service_type=None, service_name=None,
                       interface=None, region_name=None):
    try:
        # Trigger an auth error so that we can throw the exception
        # we always have
        session.get_endpoint(
            service_type=service_type,
            service_name=service_name,
            interface=interface,
            region_name=region_name)
    except catalog.EndpointNotFound:
        service_type = LEGACY_DEFAULT_SERVICE_TYPE
        try:
            session.get_endpoint(
                service_type=service_type,
                service_name=service_name,
                interface=interface,
                region_name=region_name)
        except Exception as e:
            raise RuntimeError(str(e))
    except Exception as e:
        raise RuntimeError(str(e))

    return service_type


def _load_session_client(session=None, endpoint_override=None, username=None,
                         project_id=None, project_name=None,
                         auth_url=None, password=None, auth_type=None,
                         insecure=None, user_domain_id=None,
                         user_domain_name=None, project_domain_id=None,
                         project_domain_name=None, auth_token=None,
                         timeout=None, service_type=None, service_name=None,
                         interface=None, region_name=None, api_version=None,
                         **kwargs):
    if not session:
        session = _load_session(
            username=username,
            project_id=project_id,
            project_name=project_name,
            auth_url=auth_url,
            password=password,
            auth_type=auth_type,
            insecure=insecure,
            user_domain_id=user_domain_id,
            user_domain_name=user_domain_name,
            project_domain_id=project_domain_id,
            project_domain_name=project_domain_name,
            auth_token=auth_token,
            timeout=timeout,
            **kwargs
        )

    if not endpoint_override:
        service_type = _load_service_type(
            session,
            service_type=service_type,
            service_name=service_name,
            interface=interface,
            region_name=region_name,
        )

    return httpclient.SessionClient(
        service_type=service_type,
        service_name=service_name,
        interface=interface,
        region_name=region_name,
        session=session,
        endpoint_override=endpoint_override,
        api_version=api_version,
    )


class Client(object):
    def __init__(self, username=None, api_key=None, project_id=None,
                 project_name=None, auth_url=None, magnum_url=None,
                 endpoint_type=None, endpoint_override=None,
                 service_type=DEFAULT_SERVICE_TYPE,
                 region_name=None, input_auth_token=None,
                 session=None, password=None, auth_type='password',
                 interface=None, service_name=None, insecure=False,
                 user_domain_id=None, user_domain_name=None,
                 project_domain_id=None, project_domain_name=None,
                 auth_token=None, timeout=600, api_version=None,
                 **kwargs):

        # We have to keep the api_key are for backwards compat, but let's
        # remove it from the rest of our code since it's not a keystone
        # concept
        if not password:
            password = api_key
        # Backwards compat for people passing in input_auth_token
        if input_auth_token:
            auth_token = input_auth_token
        # Backwards compat for people passing in endpoint_type
        if endpoint_type:
            interface = endpoint_type

        # osc sometimes give 'None' value
        if not interface:
            interface = 'public'

        if interface.endswith('URL'):
            interface = interface[:-3]

        # fix (yolanda): os-cloud-config is using endpoint_override
        # instead of magnum_url
        if magnum_url and not endpoint_override:
            endpoint_override = magnum_url

        if endpoint_override and auth_token:
            self.http_client = httpclient.HTTPClient(
                endpoint_override,
                token=auth_token,
                api_version=api_version,
                timeout=timeout,
                insecure=insecure,
                **kwargs
            )
        else:
            self.http_client = _load_session_client(
                session=session,
                endpoint_override=endpoint_override,
                username=username,
                project_id=project_id,
                project_name=project_name,
                auth_url=auth_url,
                password=password,
                auth_type=auth_type,
                insecure=insecure,
                user_domain_id=user_domain_id,
                user_domain_name=user_domain_name,
                project_domain_id=project_domain_id,
                project_domain_name=project_domain_name,
                auth_token=auth_token,
                timeout=timeout,
                service_type=service_type,
                service_name=service_name,
                interface=interface,
                region_name=region_name,
                api_version=api_version,
                **kwargs
            )

        self.clusters = clusters.ClusterManager(self.http_client)
        self.certificates = certificates.CertificateManager(self.http_client)
        self.cluster_templates = \
            cluster_templates.ClusterTemplateManager(self.http_client)
        self.mservices = mservices.MServiceManager(self.http_client)

        profile = kwargs.pop("profile", None)
        if profiler and profile:
            # Initialize the root of the future trace: the created trace ID
            # will be used as the very first parent to which all related
            # traces will be bound to. The given HMAC key must correspond to
            # the one set in magnum-api magnum.conf, otherwise the latter
            # will fail to check the request signature and will skip
            # initialization of osprofiler on the server side.
            profiler.init(profile)
        self.stats = stats.StatsManager(self.http_client)
        self.quotas = quotas.QuotasManager(self.http_client)
        self.nodegroups = nodegroups.NodeGroupManager(self.http_client)
        self.credentials = credentials.CredentialManager(self.http_client)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import cliutils as utils
from magnumclient.common import utils as magnum_utils
from magnumclient.exceptions import InvalidAttribute
from magnumclient.i18n import _
from magnumclient.v1 import basemodels

# Maps old parameter names to their new names and whether they are required
DEPRECATING_PARAMS = {
    "--external-network-id": "--external-network",
    "--flavor-id": "--flavor",
    "--image-id": "--image",
    "--keypair-id": "--keypair",
    "--master-flavor-id": "--master-flavor",
}


def _show_cluster_template(cluster_template):
    del cluster_template._info['links']
    utils.print_dict(cluster_template._info)


@utils.deprecation_map(DEPRECATING_PARAMS)
@utils.arg('positional_name',
           metavar='<name>',
           nargs='?',
           default=None,
           help=_('Name of the cluster template to create.'))
@utils.arg('--name',
           metavar='<name>',
           default=None,
           help=(_('Name of the cluster template to create. %s') %
                 utils.NAME_DEPRECATION_HELP))
@utils.arg('--image-id',
           dest='image',
           required=True,
           metavar='<image>',
           help=utils.deprecation_message(
               'The name or UUID of the base image to customize for the '
               'Cluster.', 'image'))
@utils.arg('--image',
           dest='image',
           required=True,
           metavar='<image>',
           help=_('The name or UUID of the base image to customize for the '
                  'Cluster.'))
@utils.arg('--keypair-id',
           dest='keypair',
           metavar='<keypair>',
           help=utils.deprecation_message(
               'The name of the SSH keypair to load into the '
               'Cluster nodes.', 'keypair'))
@utils.arg('--keypair',
           dest='keypair',
           metavar='<keypair>',
           help=_('The name of the SSH keypair to load into the '
                  'Cluster nodes.'))
@utils.arg('--external-network-id',
           dest='external_network',
           required=True,
           metavar='<external-network>',
           help=utils.deprecation_message(
                'The external Neutron network name or UUID to connect to '
                'this Cluster Template.', 'external-network'))
@utils.arg('--external-network',
           dest='external_network',
           required=True,
           metavar='<external-network>',
           help=_('The external Neutron network name or UUID to connect to '
                  'this Cluster Template.'))
@utils.arg('--coe',
           required=True,
           metavar='<coe>',
           help=_('Specify the Container Orchestration Engine to use.'))
@utils.arg('--fixed-network',
           metavar='<fixed-network>',
           help=_('The private Neutron network name to connect to this Cluster'
                  ' model.'))
@utils.arg('--fixed-subnet',
           metavar='<fixed-subnet>',
           help=_('The private Neutron subnet name to connect to Cluster.'))
@utils.arg('--network-driver',
           metavar='<network-driver>',
           help=_('The network driver name for instantiating container'
                  ' networks.'))
@utils.arg('--volume-driver',
           metavar='<volume-driver>',
           help=_('The volume driver name for instantiating container'
                  ' volume.'))
@utils.arg('--dns-nameserver',
           metavar='<dns-nameserver>',
           default='8.8.8.8',
           help=_('The DNS nameserver to use for this cluster template.'))
@utils.arg('--flavor-id',
           dest='flavor',
           metavar='<flavor>',
           default='m1.medium',
           help=utils.deprecation_message(
                'The nova flavor name or UUID to use when launching the '
                'Cluster.', 'flavor'))
@utils.arg('--flavor',
           dest='flavor',
           metavar='<flavor>',
           default='m1.medium',
           help=_('The nova flavor name or UUID to use when launching the '
                  'Cluster.'))
@utils.arg('--master-flavor-id',
           dest='master_flavor',
           metavar='<master-flavor>',
           help=utils.deprecation_message(
                'The nova flavor name or UUID to use when launching the master'
                ' node of the Cluster.', 'master-flavor'))
@utils.arg('--master-flavor',
           dest='master_flavor',
           metavar='<master-flavor>',
           help=_('The nova flavor name or UUID to use when launching the'
                  ' master node of the Cluster.'))
@utils.arg('--docker-volume-size',
           metavar='<docker-volume-size>',
           type=int,
           help=_('Specify the number of size in GB '
                  'for the docker volume to use.'))
@utils.arg('--docker-storage-driver',
           metavar='<docker-storage-driver>',
           default='devicemapper',
           help=_('Select a docker storage driver. Supported: devicemapper, '
                  'overlay. Default: devicemapper'))
@utils.arg('--http-proxy',
           metavar='<http-proxy>',
           help=_('The http_proxy address to use for nodes in Cluster.'))
@utils.arg('--https-proxy',
           metavar='<https-proxy>',
           help=_('The https_proxy address to use for nodes in Cluster.'))
@utils.arg('--no-proxy',
           metavar='<no-proxy>',
           help=_('The no_proxy address to use for nodes in Cluster.'))
@utils.arg('--labels', metavar='<KEY1=VALUE1,KEY2=VALUE2;KEY3=VALUE3...>',
           action='append', default=[],
           help=_('Arbitrary labels in the form of key=value pairs '
                  'to associate with a cluster template. '
                  'May be used multiple times.'))
@utils.arg('--tls-disabled',
           action='store_true', default=False,
           help=_('Disable TLS in the Cluster.'))
@utils.arg('--public',
           action='store_true', default=False,
           help=_('Make cluster template public.'))
@utils.arg('--registry-enabled',
           action='store_true', default=False,
           help=_('Enable docker registry in the Cluster'))
@utils.arg('--server-type',
           metavar='<server-type>',
           default='vm',
           help=_('Specify the server type to be used '
                  'for example vm. For this release '
                  'default server type will be vm.'))
@utils.arg('--master-lb-enabled',
           action='store_true', default=False,
           help=_('Indicates whether created Clusters should have a load '
                  'balancer for master nodes or not.'))
@utils.arg('--floating-ip-enabled',
           action='append_const',
           const=True,
           default=[],
           dest='floating_ip_enabled',
           help=_('Indicates whether created Clusters should have a '
                  'floating ip.'))
@utils.arg('--floating-ip-disabled',
           action='append_const',
           const=False,
           default=[],
           dest='floating_ip_enabled',
           help=_('Disables floating ip creation on the new Cluster'))
@utils.arg('--insecure-registry',
           metavar='<insecure-registry>',
           help='url of docker registry')
@utils.arg('--hidden',
           action='store_true', default=False,
           help=_('Make cluster template hidden.'))
@utils.arg('--visible',
           dest='hidden',
           action='store_false',
           help=_('Make cluster template visible.'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_template_create(cs, args):
    """Create a cluster template."""
    args.command = 'cluster-template-create'

    utils.validate_name_args(args.positional_name, args.name)

    opts = {}
    opts['name'] = args.positional_name or args.name
    opts['flavor_id'] = args.flavor
    opts['master_flavor_id'] = args.master_flavor
    opts['image_id'] = args.image
    opts['keypair_id'] = args.keypair
    opts['external_network_id'] = args.external_network
    opts['fixed_network'] = args.fixed_network
    opts['fixed_subnet'] = args.fixed_subnet
    opts['network_driver'] = args.network_driver
    opts['volume_driver'] = args.volume_driver
    opts['dns_nameserver'] = args.dns_nameserver
    opts['docker_volume_size'] = args.docker_volume_size
    opts['docker_storage_driver'] = args.docker_storage_driver
    opts['coe'] = args.coe
    opts['http_proxy'] = args.http_proxy
    opts['https_proxy'] = args.https_proxy
    opts['no_proxy'] = args.no_proxy
    opts['labels'] = magnum_utils.handle_labels(args.labels)
    opts['tls_disabled'] = args.tls_disabled
    opts['public'] = args.public
    opts['registry_enabled'] = args.registry_enabled
    opts['server_type'] = args.server_type
    opts['master_lb_enabled'] = args.master_lb_enabled
    opts['insecure_registry'] = args.insecure_registry
    opts['hidden'] = args.hidden

    if len(args.floating_ip_enabled) > 1:
        raise InvalidAttribute('--floating-ip-enabled and '
                               '--floating-ip-disabled are '
                               'mutually exclusive and '
                               'should be specified only once.')
    elif len(args.floating_ip_enabled) == 1:
        opts['floating_ip_enabled'] = args.floating_ip_enabled[0]

    cluster_template = cs.cluster_templates.create(**opts)
    _show_cluster_template(cluster_template)


@utils.arg('cluster_templates',
           metavar='<cluster_templates>',
           nargs='+',
           help=_('ID or name of the (cluster template)s to delete.'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_template_delete(cs, args):
    """Delete specified cluster template."""
    for cluster_template in args.cluster_templates:
        try:
            cs.cluster_templates.delete(cluster_template)
            print("Request to delete cluster template %s has been accepted." %
                  cluster_template)
        except Exception as e:
            print("Delete for cluster template "
                  "%(cluster_template)s failed: %(e)s" %
                  {'cluster_template': cluster_template, 'e': e})


@utils.arg('cluster_template',
           metavar='<cluster_template>',
           help=_('ID or name of the cluster template to show.'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_template_show(cs, args):
    """Show details about the given cluster template."""
    cluster_template = cs.cluster_templates.get(args.cluster_template)
    _show_cluster_template(cluster_template)


@utils.arg('--limit',
           metavar='<limit>',
           type=int,
           help=_('Maximum number of cluster templates to return'))
@utils.arg('--sort-key',
           metavar='<sort-key>',
           help=_('Column to sort results by'))
@utils.arg('--sort-dir',
           metavar='<sort-dir>',
           choices=['desc', 'asc'],
           help=_('Direction to sort. "asc" or "desc".'))
@utils.arg('--fields',
           default=None,
           metavar='<fields>',
           help=_('Comma-separated list of fields to display. '
                  'Available fields: uuid, name, coe, image_id, public, link, '
                  'apiserver_port, server_type, tls_disabled, registry_enabled'
                  )
           )
@utils.arg('--detail',
           action='store_true', default=False,
           help=_('Show detailed information about the cluster templates.')
           )
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_template_list(cs, args):
    """Print a list of cluster templates."""
    nodes = cs.cluster_templates.list(limit=args.limit,
                                      sort_key=args.sort_key,
                                      sort_dir=args.sort_dir,
                                      detail=args.detail)
    if args.detail:
        columns = basemodels.OUTPUT_ATTRIBUTES
    else:
        columns = ['uuid', 'name']
    columns += utils._get_list_table_columns_and_formatters(
        args.fields, nodes,
        exclude_fields=(c.lower() for c in columns))[0]
    utils.print_list(nodes, columns,
                     {'versions': magnum_utils.print_list_field('versions')},
                     sortby_index=None)


@utils.arg('cluster_template',
           metavar='<cluster_template>',
           help=_("UUID or name of cluster template"))
@utils.arg(
    'op',
    metavar='<op>',
    choices=['add', 'replace', 'remove'],
    help=_("Operations: 'add', 'replace' or 'remove'"))
@utils.arg(
    'attributes',
    metavar='<path=value>',
    nargs='+',
    action='append',
    default=[],
    help=_("Attributes to add/replace or remove "
           "(only PATH is necessary on remove)"))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_template_update(cs, args):
    """Updates one or more cluster template attributes."""
    patch = magnum_utils.args_array_to_patch(args.op, args.attributes[0])

    cluster_template = cs.cluster_templates.update(args.cluster_template,
                                                   patch)
    _show_cluster_template(cluster_template)



# -*- coding: utf-8 -*-
#
# Copyright 2013 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import builtins
import collections
from unittest import mock

from oslo_serialization import jsonutils
import tempfile

from magnumclient.common import cliutils
from magnumclient.common import utils
from magnumclient import exceptions as exc
from magnumclient.tests import utils as test_utils


class CommonFiltersTest(test_utils.BaseTestCase):
    def test_limit(self):
        result = utils.common_filters(limit=42)
        self.assertEqual(['limit=42'], result)

    def test_limit_0(self):
        result = utils.common_filters(limit=0)
        self.assertEqual(['limit=0'], result)

    def test_limit_negative_number(self):
        result = utils.common_filters(limit=-2)
        self.assertEqual(['limit=-2'], result)

    def test_other(self):
        for key in ('marker', 'sort_key', 'sort_dir'):
            result = utils.common_filters(**{key: 'test'})
            self.assertEqual(['%s=test' % key], result)


class SplitAndDeserializeTest(test_utils.BaseTestCase):

    def test_split_and_deserialize(self):
        ret = utils.split_and_deserialize('str=foo')
        self.assertEqual(('str', 'foo'), ret)

        ret = utils.split_and_deserialize('int=1')
        self.assertEqual(('int', 1), ret)

        ret = utils.split_and_deserialize('bool=false')
        self.assertEqual(('bool', False), ret)

        ret = utils.split_and_deserialize('list=[1, "foo", 2]')
        self.assertEqual(('list', [1, "foo", 2]), ret)

        ret = utils.split_and_deserialize('dict={"foo": 1}')
        self.assertEqual(('dict', {"foo": 1}), ret)

        ret = utils.split_and_deserialize('str_int="1"')
        self.assertEqual(('str_int', "1"), ret)

    def test_split_and_deserialize_fail(self):
        self.assertRaises(exc.CommandError,
                          utils.split_and_deserialize, 'foo:bar')


class ArgsArrayToPatchTest(test_utils.BaseTestCase):

    def test_args_array_to_patch(self):
        my_args = {
            'attributes': ['str=foo', 'int=1', 'bool=true',
                           'list=[1, 2, 3]', 'dict={"foo": "bar"}'],
            'op': 'add',
        }
        patch = utils.args_array_to_patch(my_args['op'],
                                          my_args['attributes'])
        self.assertEqual([{'op': 'add', 'value': 'foo', 'path': '/str'},
                          {'op': 'add', 'value': 1, 'path': '/int'},
                          {'op': 'add', 'value': True, 'path': '/bool'},
                          {'op': 'add', 'value': [1, 2, 3], 'path': '/list'},
                          {'op': 'add', 'value': {"foo": "bar"},
                           'path': '/dict'}], patch)

    def test_args_array_to_patch_format_error(self):
        my_args = {
            'attributes': ['foobar'],
            'op': 'add',
        }
        self.assertRaises(exc.CommandError, utils.args_array_to_patch,
                          my_args['op'], my_args['attributes'])

    def test_args_array_to_patch_remove(self):
        my_args = {
            'attributes': ['/foo', 'extra/bar'],
            'op': 'remove',
        }
        patch = utils.args_array_to_patch(my_args['op'],
                                          my_args['attributes'])
        self.assertEqual([{'op': 'remove', 'path': '/foo'},
                          {'op': 'remove', 'path': '/extra/bar'}], patch)

    def test_args_array_to_patch_invalid_op(self):
        my_args = {
            'attributes': ['/foo', 'extra/bar'],
            'op': 'invalid',
        }
        self.assertRaises(exc.CommandError, utils.args_array_to_patch,
                          my_args['op'], my_args['attributes'])


class FormatLabelsTest(test_utils.BaseTestCase):

    def test_format_label_none(self):
        self.assertEqual({}, utils.format_labels(None))

    def test_format_labels(self):
        la = utils.format_labels([
            'K1=V1,K2=V2,'
            'K3=V3,K4=V4,'
            'K5=V5'])
        self.assertEqual({'K1': 'V1',
                          'K2': 'V2',
                          'K3': 'V3',
                          'K4': 'V4',
                          'K5': 'V5'
                          }, la)

    def test_format_labels_semicolon(self):
        la = utils.format_labels([
            'K1=V1;K2=V2;'
            'K3=V3;K4=V4;'
            'K5=V5'])
        self.assertEqual({'K1': 'V1',
                          'K2': 'V2',
                          'K3': 'V3',
                          'K4': 'V4',
                          'K5': 'V5'
                          }, la)

    def test_format_labels_mix_commas_semicolon(self):
        la = utils.format_labels([
            'K1=V1,K2=V2,'
            'K3=V3;K4=V4,'
            'K5=V5'])
        self.assertEqual({'K1': 'V1',
                          'K2': 'V2',
                          'K3': 'V3',
                          'K4': 'V4',
                          'K5': 'V5'
                          }, la)

    def test_format_labels_split(self):
        la = utils.format_labels([
            'K1=V1,'
            'K2=V22222222222222222222222222222'
            '222222222222222222222222222,'
            'K3=3.3.3.3'])
        self.assertEqual({'K1': 'V1',
                          'K2': 'V22222222222222222222222222222'
                          '222222222222222222222222222',
                          'K3': '3.3.3.3'}, la)

    def test_format_labels_multiple(self):
        la = utils.format_labels([
            'K1=V1',
            'K2=V22222222222222222222222222222'
            '222222222222222222222222222',
            'K3=3.3.3.3'])
        self.assertEqual({'K1': 'V1',
                          'K2': 'V22222222222222222222222222222'
                          '222222222222222222222222222',
                          'K3': '3.3.3.3'}, la)

    def test_format_labels_multiple_colon_values(self):
        la = utils.format_labels([
            'K1=V1',
            'K2=V2,V22,V222,V2222',
            'K3=3.3.3.3'])
        self.assertEqual({'K1': 'V1',
                          'K2': 'V2,V22,V222,V2222',
                          'K3': '3.3.3.3'}, la)

    def test_format_labels_parse_comma_false(self):
        la = utils.format_labels(
            ['K1=V1,K2=2.2.2.2,K=V'],
            parse_comma=False)
        self.assertEqual({'K1': 'V1,K2=2.2.2.2,K=V'}, la)

    def test_format_labels_multiple_values_per_labels(self):
        la = utils.format_labels([
            'K1=V1',
            'K1=V2'])
        self.assertEqual({'K1': 'V1,V2'}, la)

    def test_format_label_special_label(self):
        labels = ['K1=V1,K22.2.2.2']
        la = utils.format_labels(
            labels,
            parse_comma=True)
        self.assertEqual({'K1': 'V1,K22.2.2.2'}, la)

    def test_format_multiple_bad_label(self):
        labels = ['K1=V1', 'K22.2.2.2']
        ex = self.assertRaises(exc.CommandError,
                               utils.format_labels, labels)
        self.assertEqual('labels must be a list of KEY=VALUE '
                         'not K22.2.2.2', str(ex))


class CliUtilsTest(test_utils.BaseTestCase):

    def test_keys_and_vals_to_strs(self):
        dict_in = {'a': '1',
                   'b': {'x': 1,
                         'y': '2',
                         'z': '3'},
                   'c': 7}

        dict_exp = collections.OrderedDict([
            ('a', '1'),
            ('b', collections.OrderedDict([
                ('x', 1),
                ('y', '2'),
                ('z', '3')])),
            ('c', 7)])

        dict_out = cliutils.keys_and_vals_to_strs(dict_in)
        dict_act = collections.OrderedDict([
            ('a', dict_out['a']),
            ('b', collections.OrderedDict(sorted(dict_out['b'].items()))),
            ('c', dict_out['c'])])

        self.assertEqual(str(dict_exp), str(dict_act))


class HandleJsonFromFileTest(test_utils.BaseTestCase):

    def test_handle_json_from_file_bad_json(self):
        contents = 'foo'
        with tempfile.NamedTemporaryFile(mode='w') as f:
            f.write(contents)
            f.flush()
            self.assertRaisesRegex(exc.InvalidAttribute,
                                   'For JSON',
                                   utils.handle_json_from_file, f.name)

    def test_handle_json_from_file_valid_file(self):
        contents = '{"step": "upgrade", "interface": "deploy"}'

        with tempfile.NamedTemporaryFile(mode='w') as f:
            f.write(contents)
            f.flush()
            steps = utils.handle_json_from_file(f.name)

        self.assertEqual(jsonutils.loads(contents), steps)

    @mock.patch.object(builtins, 'open', autospec=True)
    def test_handle_json_from_file_open_fail(self, mock_open):
        mock_file_object = mock.MagicMock()
        mock_file_handle = mock.MagicMock()
        mock_file_handle.__enter__.return_value = mock_file_object
        mock_open.return_value = mock_file_handle
        mock_file_object.read.side_effect = IOError

        with tempfile.NamedTemporaryFile(mode='w') as f:
            self.assertRaisesRegex(exc.InvalidAttribute,
                                   "from file",
                                   utils.handle_json_from_file, f.name)
            mock_open.assert_called_once_with(f.name, 'r')
            mock_file_object.read.assert_called_once_with()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import testtools
from testtools import matchers

from magnumclient.tests import utils
from magnumclient.v1 import quotas


QUOTA1 = {
    'id': 123,
    'resource': "Cluster",
    'hard_limit': 5,
    'project_id': 'abc'
}

QUOTA2 = {
    'id': 124,
    'resource': "Cluster",
    'hard_limit': 10,
    'project_id': 'bcd'
}

CREATE_QUOTA = copy.deepcopy(QUOTA1)
del CREATE_QUOTA['id']

UPDATED_QUOTA = copy.deepcopy(QUOTA2)
NEW_HARD_LIMIT = 20
UPDATED_QUOTA['hard_limit'] = NEW_HARD_LIMIT

fake_responses = {
    '/v1/quotas?all_tenants=True':
    {
        'GET': (
            {},
            {'quotas': [QUOTA1, QUOTA2]},
        ),
    },
    '/v1/quotas':
    {
        'GET': (
            {},
            {'quotas': [QUOTA1]},
        ),
        'POST': (
            {},
            QUOTA1,
        ),
    },
    '/v1/quotas/%(id)s/%(res)s' % {'id': QUOTA2['project_id'],
                                   'res': QUOTA2['resource']}:
    {
        'GET': (
            {},
            QUOTA2,
        ),
        'PATCH': (
            {},
            UPDATED_QUOTA,
        ),
        'DELETE': (
            {},
            None,
        ),
    },
}


class QuotasManagerTest(testtools.TestCase):

    def setUp(self):
        super(QuotasManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = quotas.QuotasManager(self.api)

    def test_list_quotas(self):
        quotas = self.mgr.list()
        expect = [
            ('GET', '/v1/quotas', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(quotas, matchers.HasLength(1))

    def test_list_quotas_all(self):
        quotas = self.mgr.list(all_tenants=True)
        expect = [
            ('GET', '/v1/quotas?all_tenants=True', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(quotas, matchers.HasLength(2))

    def test_show_project_resource_quota(self):
        expect = [
            ('GET',
             '/v1/quotas/%(id)s/%(res)s' % {'id': QUOTA2['project_id'],
                                            'res': QUOTA2['resource']},
             {},
             None),
        ]
        quotas = self.mgr.get(QUOTA2['project_id'], QUOTA2['resource'])
        self.assertEqual(expect, self.api.calls)
        expected_quotas = QUOTA2
        self.assertEqual(expected_quotas, quotas._info)

    def test_quota_create(self):
        quota = self.mgr.create(**CREATE_QUOTA)
        expect = [
            ('POST', '/v1/quotas', {}, CREATE_QUOTA),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(QUOTA1, quota._info)

    def test_quota_update(self):
        patch = {
            'resource': "Cluster",
            'hard_limit': NEW_HARD_LIMIT,
            'project_id': 'bcd'
        }
        quota = self.mgr.update(id=QUOTA2['project_id'],
                                resource=QUOTA2['resource'],
                                patch=patch)
        expect = [
            ('PATCH', '/v1/quotas/%(id)s/%(res)s' % {
                'id': QUOTA2['project_id'],
                'res': QUOTA2['resource']}, {}, patch),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(NEW_HARD_LIMIT, quota.hard_limit)

    def test_quota_delete(self):
        quota = self.mgr.delete(QUOTA2['project_id'], QUOTA2['resource'])
        expect = [
            ('DELETE',
             '/v1/quotas/%(id)s/%(res)s' % {'id': QUOTA2['project_id'],
                                            'res': QUOTA2['resource']},
             {},
             None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertIsNone(quota)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from magnumclient.i18n import _

from osc_lib.command import command


def _show_cert(certificate):
    try:
        print(certificate.pem)
    except AttributeError:
        return None


def _get_target_uuid(cs, args):
    target = None
    if args.cluster:
        target = cs.clusters.get(args.cluster)
    return target.uuid


class RotateCa(command.Command):
    _description = _("Rotate the CA certificate for cluster to revoke access.")

    def get_parser(self, prog_name):
        parser = super(RotateCa, self).get_parser(prog_name)
        parser.add_argument('cluster',
                            metavar='<cluster>',
                            help='ID or name of the cluster')
        return parser

    def take_action(self, parsed_args):
        mag_client = self.app.client_manager.container_infra
        cluster = mag_client.clusters.get(parsed_args.cluster)
        opts = {
            'cluster_uuid': cluster.uuid
        }

        mag_client.certificates.rotate_ca(**opts)
        print("Request to rotate the CA certificate for cluster %s "
              "has been accepted." % cluster.uuid)


class ShowCa(command.Command):
    _description = _("Show details about the CA certificate for a cluster.")

    def get_parser(self, prog_name):
        parser = super(ShowCa, self).get_parser(prog_name)
        # NOTE: All arguments are positional and, if not provided
        # with a default, required.
        parser.add_argument('cluster',
                            metavar='<cluster>',
                            help='ID or name of the cluster')
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra
        cluster = mag_client.clusters.get(parsed_args.cluster)
        cert = mag_client.certificates.get(cluster.uuid)
        _show_cert(cert)


class SignCa(command.Command):
    _description = _("Generate the CA certificate for a cluster.")

    def get_parser(self, prog_name):
        parser = super(SignCa, self).get_parser(prog_name)
        # NOTE: All arguments are positional and, if not provided
        # with a default, required.
        parser.add_argument('cluster',
                            metavar='<cluster>',
                            help='ID or name of the cluster')
        parser.add_argument('csr',
                            metavar='<csr>',
                            help='File path of csr file to send to Magnum'
                                 ' to get signed.')
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        mag_client = self.app.client_manager.container_infra

        opts = {
            'cluster_uuid': _get_target_uuid(mag_client, parsed_args)
        }

        if parsed_args.csr is None or not os.path.isfile(parsed_args.csr):
            print('A CSR must be provided.')
            return

        with open(parsed_args.csr, 'r') as f:
            opts['csr'] = f.read()

        cert = mag_client.certificates.create(**opts)
        _show_cert(cert)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from osc_lib.command import command
from osc_lib import utils


def _get_client(obj, parsed_args):
    obj.log.debug("take_action(%s)" % parsed_args)
    return obj.app.client_manager.container_infra


class ListService(command.Lister):
    """Print a list of Magnum services."""

    log = logging.getLogger(__name__ + ".ListService")

    def get_parser(self, prog_name):
        parser = super(ListService, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        services = client.mservices.list()
        columns = ('id', 'host', 'binary', 'state', 'disabled',
                   'disabled_reason', 'created_at', 'updated_at')
        return (columns, (utils.get_item_properties(service, columns)
                          for service in services))



# Copyright (c) 2015 Thales Services SAS
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from unittest import mock

from keystoneauth1.exceptions import catalog

from magnumclient.v1 import client


class ClientInitializeTest(testtools.TestCase):

    def _load_session_kwargs(self):
        return {
            'username': None,
            'project_id': None,
            'project_name': None,
            'auth_url': None,
            'password': None,
            'auth_type': 'password',
            'insecure': False,
            'user_domain_id': None,
            'user_domain_name': None,
            'project_domain_id': None,
            'project_domain_name': None,
            'auth_token': None,
            'timeout': 600,
        }

    def _load_service_type_kwargs(self):
        return {
            'interface': 'public',
            'region_name': None,
            'service_name': None,
            'service_type': 'container-infra',
        }

    def _session_client_kwargs(self, session):
        kwargs = self._load_service_type_kwargs()
        kwargs['endpoint_override'] = None
        kwargs['session'] = session
        kwargs['api_version'] = None

        return kwargs

    @mock.patch('magnumclient.common.httpclient.SessionClient')
    @mock.patch('magnumclient.v1.client._load_session')
    @mock.patch('magnumclient.v1.client._load_service_type',
                return_value='container-infra')
    def test_init_with_session(self,
                               mock_load_service_type,
                               mock_load_session,
                               mock_http_client):
        session = mock.Mock()
        client.Client(session=session)
        mock_load_session.assert_not_called()
        mock_load_service_type.assert_called_once_with(
            session,
            **self._load_service_type_kwargs()
        )
        mock_http_client.assert_called_once_with(
            **self._session_client_kwargs(session)
        )

    def _test_init_with_secret(self,
                               init_func,
                               mock_load_service_type,
                               mock_load_session,
                               mock_http_client,):
        expected_password = 'expected_password'
        session = mock.Mock()
        mock_load_session.return_value = session
        init_func(expected_password)
        load_session_args = self._load_session_kwargs()
        load_session_args['password'] = expected_password
        mock_load_session.assert_called_once_with(
            **load_session_args
        )
        mock_load_service_type.assert_called_once_with(
            session,
            **self._load_service_type_kwargs()
        )
        mock_http_client.assert_called_once_with(
            **self._session_client_kwargs(session)
        )

    @mock.patch('magnumclient.common.httpclient.SessionClient')
    @mock.patch('magnumclient.v1.client._load_session')
    @mock.patch('magnumclient.v1.client._load_service_type',
                return_value='container-infra')
    def test_init_with_password(self,
                                mock_load_service_type,
                                mock_load_session,
                                mock_http_client):
        self._test_init_with_secret(
            lambda x: client.Client(password=x),
            mock_load_service_type,
            mock_load_session,
            mock_http_client
        )

    @mock.patch('magnumclient.common.httpclient.SessionClient')
    @mock.patch('magnumclient.v1.client._load_session')
    @mock.patch('magnumclient.v1.client._load_service_type',
                return_value='container-infra')
    def test_init_with_api_key(self,
                               mock_load_service_type,
                               mock_load_session,
                               mock_http_client):
        self._test_init_with_secret(
            lambda x: client.Client(api_key=x),
            mock_load_service_type,
            mock_load_session,
            mock_http_client
        )

    @mock.patch('magnumclient.common.httpclient.HTTPClient')
    def test_init_with_auth_token(self,
                                  mock_http_client,):
        expected_token = 'expected_password'
        expected_magnum_url = 'expected_magnum_url'
        expected_api_version = 'expected_api_version'
        expected_insecure = False
        expected_timeout = 600
        expected_kwargs = {'expected_key': 'expected_value'}
        client.Client(auth_token=expected_token,
                      magnum_url=expected_magnum_url,
                      api_version=expected_api_version,
                      timeout=expected_timeout,
                      insecure=expected_insecure,
                      **expected_kwargs)

        mock_http_client.assert_called_once_with(
            expected_magnum_url,
            token=expected_token,
            api_version=expected_api_version,
            timeout=expected_timeout,
            insecure=expected_insecure,
            **expected_kwargs)

    def _test_init_with_interface(self,
                                  init_func,
                                  mock_load_service_type,
                                  mock_load_session,
                                  mock_http_client):
        expected_interface = 'admin'
        session = mock.Mock()
        mock_load_session.return_value = session
        init_func(expected_interface)
        mock_load_session.assert_called_once_with(
            **self._load_session_kwargs()
        )
        expected_kwargs = self._load_service_type_kwargs()
        expected_kwargs['interface'] = expected_interface
        mock_load_service_type.assert_called_once_with(
            session,
            **expected_kwargs
        )
        expected_kwargs = self._session_client_kwargs(session)
        expected_kwargs['interface'] = expected_interface
        mock_http_client.assert_called_once_with(
            **expected_kwargs
        )

    @mock.patch('magnumclient.common.httpclient.SessionClient')
    @mock.patch('magnumclient.v1.client._load_session')
    @mock.patch('magnumclient.v1.client._load_service_type',
                return_value='container-infra')
    def test_init_with_interface(self,
                                 mock_load_service_type,
                                 mock_load_session,
                                 mock_http_client):
        self._test_init_with_interface(
            lambda x: client.Client(interface=x),
            mock_load_service_type,
            mock_load_session,
            mock_http_client
        )

    @mock.patch('magnumclient.common.httpclient.SessionClient')
    @mock.patch('magnumclient.v1.client._load_session')
    @mock.patch('magnumclient.v1.client._load_service_type',
                return_value='container-infra')
    def test_init_with_endpoint_type(self,
                                     mock_load_service_type,
                                     mock_load_session,
                                     mock_http_client):
        self._test_init_with_interface(
            lambda x: client.Client(interface='public',
                                    endpoint_type=('%sURL' % x)),
            mock_load_service_type,
            mock_load_session,
            mock_http_client
        )

    @mock.patch('magnumclient.common.httpclient.SessionClient')
    @mock.patch('magnumclient.v1.client._load_session')
    def test_init_with_legacy_service_type(self,
                                           mock_load_session,
                                           mock_http_client):
        session = mock.Mock()
        mock_load_session.return_value = session
        session.get_endpoint.side_effect = [
            catalog.EndpointNotFound(),
            mock.Mock()
        ]
        client.Client(username='myuser', auth_url='authurl')
        expected_kwargs = self._session_client_kwargs(session)
        expected_kwargs['service_type'] = 'container'
        mock_http_client.assert_called_once_with(
            **expected_kwargs
        )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from magnumclient.tests import utils
from magnumclient.v1 import stats


CLUSTER1 = {'id': 123,
            'uuid': '66666666-7777-8888-9999-000000000001',
            'name': 'cluster1',
            'cluster_template_id': 'e74c40e0-d825-11e2-a28f-0800200c9a61',
            'stack_id': '5d12f6fd-a196-4bf0-ae4c-1f639a523a51',
            'api_address': '172.17.2.1',
            'node_addresses': ['172.17.2.3'],
            'node_count': 2,
            'master_count': 1,
            'project_id': 'abc'
            }
CLUSTER2 = {'id': 124,
            'uuid': '66666666-7777-8888-9999-000000000002',
            'name': 'cluster2',
            'cluster_template_id': 'e74c40e0-d825-11e2-a28f-0800200c9a62',
            'stack_id': '5d12f6fd-a196-4bf0-ae4c-1f639a523a52',
            'api_address': '172.17.2.2',
            'node_addresses': ['172.17.2.4'],
            'node_count': 2,
            'master_count': 1,
            'project_id': 'bcd'
            }


nc = 'node_count'
mc = 'master_count'
C1 = CLUSTER1
C2 = CLUSTER2
fake_responses = {
    '/v1/stats':
    {
        'GET': (
            {},
            {'clusters': 2, 'nodes': C1[nc] + C1[mc] + C2[nc] + C2[mc]},
        )
    },
    '/v1/stats?project_id=%s' % C2['project_id']:
    {
        'GET': (
            {},
            {'clusters': 1, 'nodes': C2[nc] + C2[mc]},
        )
    },
}


class StatsManagerTest(testtools.TestCase):

    def setUp(self):
        super(StatsManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = stats.StatsManager(self.api)

    def test_stats(self):
        stats = self.mgr.list()
        expect = [
            ('GET', '/v1/stats', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        expected_stats = {'clusters': 2,
                          'nodes': C1[nc] + C1[mc] + C2[nc] + C2[mc]}
        self.assertEqual(expected_stats, stats._info)

    def test_stats_with_project_id(self):
        expect = [
            ('GET', '/v1/stats?project_id=%s' % CLUSTER2['project_id'], {},
             None),
        ]
        stats = self.mgr.list(project_id=CLUSTER2['project_id'])
        self.assertEqual(expect, self.api.calls)
        expected_stats = {'clusters': 1,
                          'nodes': C2[nc] + C2[mc]}
        self.assertEqual(expected_stats, stats._info)



# Copyright 2015 Rackspace, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import base
from magnumclient import exceptions


CREATION_ATTRIBUTES = ['cluster_uuid', 'csr']


class Certificate(base.Resource):
    def __repr__(self):
        return "<Certificate %s>" % self._info


class CertificateManager(base.Manager):
    resource_class = Certificate

    @staticmethod
    def _path(id=None):
        return '/v1/certificates/%s' % id if id else '/v1/certificates'

    def get(self, cluster_uuid):
        try:
            return self._list(self._path(cluster_uuid))[0]
        except IndexError:
            return None

    def create(self, **kwargs):
        new = {}
        for (key, value) in kwargs.items():
            if key in CREATION_ATTRIBUTES:
                new[key] = value
            else:
                raise exceptions.InvalidAttribute(
                    "Key must be in %s" % ",".join(CREATION_ATTRIBUTES))
        return self._create(self._path(), new)

    def rotate_ca(self, **kwargs):
        return self._update(self._path(id=kwargs['cluster_uuid']))



# Copyright (c) 2015 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools
from unittest import mock

from magnumclient import client


class ClientTest(testtools.TestCase):

    @mock.patch('magnumclient.v1.client.Client')
    def test_no_version_argument(self, mock_magnum_client):
        client.Client(auth_token='mytoken', magnum_url='http://myurl/')
        mock_magnum_client.assert_called_with(
            auth_token='mytoken', magnum_url='http://myurl/')

    @mock.patch('magnumclient.v1.client.Client')
    def test_valid_version_argument(self, mock_magnum_client):
        client.Client(version='1', magnum_url='http://myurl/')
        mock_magnum_client.assert_called_with(magnum_url='http://myurl/')

    @mock.patch('magnumclient.v1.client.Client')
    def test_invalid_version_argument(self, mock_magnum_client):
        self.assertRaises(
            ValueError,
            client.Client, version='2', magnum_url='http://myurl/')



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import cliutils as utils
from magnumclient.i18n import _

from osc_lib.command import command


class ListStats(command.Command):
    _description = _("Show stats for the given project_id")

    def get_parser(self, prog_name):
        parser = super(ListStats, self).get_parser(prog_name)
        parser.add_argument('project_id',
                            metavar='<project>',
                            help='Project ID')
        return parser

    def take_action(self, parsed_args):
        mag_client = self.app.client_manager.container_infra
        opts = {
            'project_id': parsed_args.project_id
        }

        stats = mag_client.stats.list(**opts)
        try:
            utils.print_dict(stats._info)
        except AttributeError:
            return None



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
from unittest import mock

from testtools import matchers

from magnumclient.tests import utils

FAKE_ENV = {'OS_USERNAME': 'username',
            'OS_PASSWORD': 'password',
            'OS_PROJECT_NAME': 'project_name',
            'OS_AUTH_URL': 'http://no.where/v2.0',
            'BYPASS_URL': 'http://magnum'}


class TestCommandLineArgument(utils.TestCase):
    _unrecognized_arg_error = [
        r'.*?^usage: ',
        r'.*?^error: unrecognized arguments:',
        r".*?^Try 'magnum help ' for more information.",
        ]

    _mandatory_group_arg_error = [
        r'.*?^usage: ',
        r'.*?^error: one of the arguments',
        r".*?^Try 'magnum help ",
    ]

    _too_many_group_arg_error = [
        r'.*?^usage: ',
        r'.*?^error: (argument \-\-[a-z\-]*: not allowed with argument )',
        r".*?^Try 'magnum help ",
    ]

    _mandatory_arg_error = [
        r'.*?^usage: ',
        r'.*?^error: (the following arguments|argument)',
        r".*?^Try 'magnum help ",
    ]

    _duplicate_arg_error = [
        r'.*?^usage: ',
        r'.*?^error: (Duplicate "<.*>" arguments:)',
        r".*?^Try 'magnum help ",
    ]

    _deprecated_warning = [
        r'.*(WARNING: The \-\-[a-z\-]* parameter is deprecated)+',
        (r'.*(Use the [\<\-a-z\-\>]* (positional )*parameter to avoid seeing '
         'this message)+')
    ]

    _few_argument_error = [
        r'.*?^usage: magnum ',
        r'.*?^error: (the following arguments|too few arguments)',
        r".*?^Try 'magnum help ",
    ]

    _invalid_value_error = [
        r'.*?^usage: ',
        r'.*?^error: argument .*: invalid .* value:',
        r".*?^Try 'magnum help ",
    ]

    def setUp(self):
        super(TestCommandLineArgument, self).setUp()
        self.make_env(fake_env=FAKE_ENV)
        session_client = mock.patch(
            'magnumclient.common.httpclient.SessionClient')
        session_client.start()
        loader = mock.patch('keystoneauth1.loading.get_plugin_loader')
        loader.start()
        session = mock.patch('keystoneauth1.session.Session')
        session.start()

        self.addCleanup(session_client.stop)
        self.addCleanup(loader.stop)
        self.addCleanup(session.stop)

    def _test_arg_success(self, command, keyword=None):
        stdout, stderr = self.shell(command)
        if keyword:
            self.assertIn(keyword, (stdout + stderr))

    def _test_arg_failure(self, command, error_msg):
        stdout, stderr = self.shell(command, (2,))
        for line in error_msg:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(line,
                            re.DOTALL | re.MULTILINE))



# Copyright 2015 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from http import client as http_client
import io
from unittest import mock

from oslo_serialization import jsonutils
import socket

from magnumclient.common.apiclient.exceptions import GatewayTimeout
from magnumclient.common.apiclient.exceptions import MultipleChoices
from magnumclient.common import httpclient as http
from magnumclient import exceptions as exc
from magnumclient.tests import utils

NORMAL_ERROR = 0
ERROR_DICT = 1
ERROR_LIST_WITH_DETAIL = 2
ERROR_LIST_WITH_DESC = 3


def _get_error_body(faultstring=None, debuginfo=None, err_type=NORMAL_ERROR):
    if err_type == NORMAL_ERROR:
        error_body = {
            'faultstring': faultstring,
            'debuginfo': debuginfo
        }
        raw_error_body = jsonutils.dumps(error_body)
        body = {'error_message': raw_error_body}
    elif err_type == ERROR_DICT:
        body = {'error': {'title': faultstring, 'message': debuginfo}}
    elif err_type == ERROR_LIST_WITH_DETAIL:
        main_body = {'title': faultstring, 'detail': debuginfo}
        body = {'errors': [main_body]}
    elif err_type == ERROR_LIST_WITH_DESC:
        main_body = {'title': faultstring, 'description': debuginfo}
        body = {'errors': [main_body]}
    raw_body = jsonutils.dumps(body)
    return raw_body


HTTP_CLASS = http_client.HTTPConnection
HTTPS_CLASS = http.VerifiedHTTPSConnection
DEFAULT_TIMEOUT = 600


class HttpClientTest(utils.BaseTestCase):

    def test_url_generation_trailing_slash_in_base(self):
        client = http.HTTPClient('http://localhost/')
        url = client._make_connection_url('/v1/resources')
        self.assertEqual('/v1/resources', url)

    def test_url_generation_without_trailing_slash_in_base(self):
        client = http.HTTPClient('http://localhost')
        url = client._make_connection_url('/v1/resources')
        self.assertEqual('/v1/resources', url)

    def test_url_generation_prefix_slash_in_path(self):
        client = http.HTTPClient('http://localhost/')
        url = client._make_connection_url('/v1/resources')
        self.assertEqual('/v1/resources', url)

    def test_url_generation_without_prefix_slash_in_path(self):
        client = http.HTTPClient('http://localhost')
        url = client._make_connection_url('v1/resources')
        self.assertEqual('/v1/resources', url)

    def test_server_exception_empty_body(self):
        error_body = _get_error_body()
        fake_resp = utils.FakeResponse({'content-type': 'application/json'},
                                       io.StringIO(error_body),
                                       version=1,
                                       status=500)
        client = http.HTTPClient('http://localhost/')
        client.get_connection = (
            lambda *a, **kw: utils.FakeConnection(fake_resp))

        error = self.assertRaises(exc.InternalServerError,
                                  client.json_request,
                                  'GET', '/v1/resources')
        self.assertEqual('Internal Server Error (HTTP 500)', str(error))

    def test_server_exception_msg_only(self):
        error_msg = 'test error msg'
        error_body = _get_error_body(error_msg, err_type=ERROR_DICT)
        fake_resp = utils.FakeResponse({'content-type': 'application/json'},
                                       io.StringIO(error_body),
                                       version=1,
                                       status=500)
        client = http.HTTPClient('http://localhost/')
        client.get_connection = (
            lambda *a, **kw: utils.FakeConnection(fake_resp))

        error = self.assertRaises(exc.InternalServerError,
                                  client.json_request,
                                  'GET', '/v1/resources')
        self.assertEqual(error_msg + ' (HTTP 500)', str(error))

    def test_server_exception_msg_and_traceback(self):
        error_msg = 'another test error'
        error_trace = ("\"Traceback (most recent call last):\\n\\n  "
                       "File \\\"/usr/local/lib/python2.7/...")
        error_body = _get_error_body(error_msg, error_trace,
                                     ERROR_LIST_WITH_DESC)
        fake_resp = utils.FakeResponse({'content-type': 'application/json'},
                                       io.StringIO(error_body),
                                       version=1,
                                       status=500)
        client = http.HTTPClient('http://localhost/')
        client.get_connection = (
            lambda *a, **kw: utils.FakeConnection(fake_resp))

        error = self.assertRaises(exc.InternalServerError,
                                  client.json_request,
                                  'GET', '/v1/resources')

        self.assertEqual(
            '%(error)s (HTTP 500)\n%(trace)s' % {'error': error_msg,
                                                 'trace': error_trace},
            "%(error)s\n%(details)s" % {'error': str(error),
                                        'details': str(error.details)})

    def test_server_exception_address(self):
        endpoint = 'https://magnum-host:6385'
        client = http.HTTPClient(endpoint, token='foobar', insecure=True,
                                 ca_file='/path/to/ca_file')
        client.get_connection = (
            lambda *a, **kw: utils.FakeConnection(exc=socket.gaierror))

        self.assertRaises(exc.EndpointNotFound, client.json_request,
                          'GET', '/v1/resources', body='farboo')

    def test_server_exception_socket(self):
        client = http.HTTPClient('http://localhost/', token='foobar')
        client.get_connection = (
            lambda *a, **kw: utils.FakeConnection(exc=socket.error))

        self.assertRaises(exc.ConnectionRefused, client.json_request,
                          'GET', '/v1/resources')

    def test_server_exception_endpoint(self):
        endpoint = 'https://magnum-host:6385'
        client = http.HTTPClient(endpoint, token='foobar', insecure=True,
                                 ca_file='/path/to/ca_file')
        client.get_connection = (
            lambda *a, **kw: utils.FakeConnection(exc=socket.gaierror))

        self.assertRaises(exc.EndpointNotFound, client.json_request,
                          'GET', '/v1/resources', body='farboo')

    def test_get_connection(self):
        endpoint = 'https://magnum-host:6385'
        client = http.HTTPClient(endpoint)
        conn = client.get_connection()
        self.assertTrue(conn, http.VerifiedHTTPSConnection)

    def test_get_connection_exception(self):
        endpoint = 'http://magnum-host:6385/'
        expected = (HTTP_CLASS,
                    ('magnum-host', 6385, ''),
                    {'timeout': DEFAULT_TIMEOUT})
        params = http.HTTPClient.get_connection_params(endpoint)
        self.assertEqual(expected, params)

    def test_get_connection_params_with_ssl(self):
        endpoint = 'https://magnum-host:6385'
        expected = (HTTPS_CLASS,
                    ('magnum-host', 6385, ''),
                    {
                        'timeout': DEFAULT_TIMEOUT,
                        'ca_file': None,
                        'cert_file': None,
                        'key_file': None,
                        'insecure': False,
                    })
        params = http.HTTPClient.get_connection_params(endpoint)
        self.assertEqual(expected, params)

    def test_get_connection_params_with_ssl_params(self):
        endpoint = 'https://magnum-host:6385'
        ssl_args = {
            'ca_file': '/path/to/ca_file',
            'cert_file': '/path/to/cert_file',
            'key_file': '/path/to/key_file',
            'insecure': True,
        }

        expected_kwargs = {'timeout': DEFAULT_TIMEOUT}
        expected_kwargs.update(ssl_args)
        expected = (HTTPS_CLASS,
                    ('magnum-host', 6385, ''),
                    expected_kwargs)
        params = http.HTTPClient.get_connection_params(endpoint, **ssl_args)
        self.assertEqual(expected, params)

    def test_get_connection_params_with_timeout(self):
        endpoint = 'http://magnum-host:6385'
        expected = (HTTP_CLASS,
                    ('magnum-host', 6385, ''),
                    {'timeout': 300.0})
        params = http.HTTPClient.get_connection_params(endpoint, timeout=300)
        self.assertEqual(expected, params)

    def test_get_connection_params_with_version(self):
        endpoint = 'http://magnum-host:6385/v1'
        expected = (HTTP_CLASS,
                    ('magnum-host', 6385, ''),
                    {'timeout': DEFAULT_TIMEOUT})
        params = http.HTTPClient.get_connection_params(endpoint)
        self.assertEqual(expected, params)

    def test_get_connection_params_with_version_trailing_slash(self):
        endpoint = 'http://magnum-host:6385/v1/'
        expected = (HTTP_CLASS,
                    ('magnum-host', 6385, ''),
                    {'timeout': DEFAULT_TIMEOUT})
        params = http.HTTPClient.get_connection_params(endpoint)
        self.assertEqual(expected, params)

    def test_get_connection_params_with_subpath(self):
        endpoint = 'http://magnum-host:6385/magnum'
        expected = (HTTP_CLASS,
                    ('magnum-host', 6385, '/magnum'),
                    {'timeout': DEFAULT_TIMEOUT})
        params = http.HTTPClient.get_connection_params(endpoint)
        self.assertEqual(expected, params)

    def test_get_connection_params_with_subpath_trailing_slash(self):
        endpoint = 'http://magnum-host:6385/magnum/'
        expected = (HTTP_CLASS,
                    ('magnum-host', 6385, '/magnum'),
                    {'timeout': DEFAULT_TIMEOUT})
        params = http.HTTPClient.get_connection_params(endpoint)
        self.assertEqual(expected, params)

    def test_get_connection_params_with_subpath_version(self):
        endpoint = 'http://magnum-host:6385/magnum/v1'
        expected = (HTTP_CLASS,
                    ('magnum-host', 6385, '/magnum'),
                    {'timeout': DEFAULT_TIMEOUT})
        params = http.HTTPClient.get_connection_params(endpoint)
        self.assertEqual(expected, params)

    def test_get_connection_params_with_subpath_version_trailing_slash(self):
        endpoint = 'http://magnum-host:6385/magnum/v1/'
        expected = (HTTP_CLASS,
                    ('magnum-host', 6385, '/magnum'),
                    {'timeout': DEFAULT_TIMEOUT})
        params = http.HTTPClient.get_connection_params(endpoint)
        self.assertEqual(expected, params)

    def test_get_connection_params_with_unsupported_scheme(self):
        endpoint = 'foo://magnum-host:6385/magnum/v1/'
        self.assertRaises(exc.EndpointException,
                          http.HTTPClient.get_connection_params, endpoint)

    def test_401_unauthorized_exception(self):
        error_body = _get_error_body(err_type=ERROR_LIST_WITH_DETAIL)
        fake_resp = utils.FakeResponse({'content-type': 'text/plain'},
                                       io.StringIO(error_body),
                                       version=1,
                                       status=401)
        client = http.HTTPClient('http://localhost/')
        client.get_connection = (lambda *a,
                                 **kw: utils.FakeConnection(fake_resp))

        self.assertRaises(exc.Unauthorized, client.json_request,
                          'GET', '/v1/resources')

    def test_server_redirect_exception(self):
        fake_redirect_resp = utils.FakeResponse(
            {'content-type': 'application/octet-stream'},
            'foo', version=1, status=301)
        fake_resp = utils.FakeResponse(
            {'content-type': 'application/octet-stream'},
            'bar', version=1, status=300)
        client = http.HTTPClient('http://localhost/')
        conn = utils.FakeConnection(fake_redirect_resp,
                                    redirect_resp=fake_resp)
        client.get_connection = (lambda *a, **kw: conn)

        self.assertRaises(MultipleChoices, client.json_request,
                          'GET', '/v1/resources')

    def test_server_body_undecode_json(self):
        err = "foo"
        fake_resp = utils.FakeResponse(
            {'content-type': 'application/json'},
            io.StringIO(err), version=1, status=200)
        client = http.HTTPClient('http://localhost/')
        conn = utils.FakeConnection(fake_resp)
        client.get_connection = (lambda *a, **kw: conn)

        resp, body = client.json_request('GET', '/v1/resources')

        self.assertEqual(resp, fake_resp)
        self.assertEqual(err, body)

    def test_server_success_body_app(self):
        fake_resp = utils.FakeResponse(
            {'content-type': 'application/octet-stream'},
            'bar', version=1, status=200)
        client = http.HTTPClient('http://localhost/')
        conn = utils.FakeConnection(fake_resp)
        client.get_connection = (lambda *a, **kw: conn)

        resp, body = client.json_request('GET', '/v1/resources')

        self.assertEqual(resp, fake_resp)
        self.assertIsNone(body)

    def test_server_success_body_none(self):
        fake_resp = utils.FakeResponse(
            {'content-type': None},
            io.StringIO('bar'), version=1, status=200)
        client = http.HTTPClient('http://localhost/')
        conn = utils.FakeConnection(fake_resp)
        client.get_connection = (lambda *a, **kw: conn)

        resp, body = client.json_request('GET', '/v1/resources')

        self.assertEqual(resp, fake_resp)
        self.assertIsInstance(body, list)

    def test_server_success_body_json(self):
        err = _get_error_body()
        fake_resp = utils.FakeResponse(
            {'content-type': 'application/json'},
            io.StringIO(err), version=1, status=200)
        client = http.HTTPClient('http://localhost/')
        conn = utils.FakeConnection(fake_resp)
        client.get_connection = (lambda *a, **kw: conn)

        resp, body = client.json_request('GET', '/v1/resources')

        self.assertEqual(resp, fake_resp)
        self.assertEqual(jsonutils.dumps(body), err)

    def test_raw_request(self):
        fake_resp = utils.FakeResponse(
            {'content-type': 'application/octet-stream'},
            'bar', version=1, status=200)
        client = http.HTTPClient('http://localhost/')
        conn = utils.FakeConnection(fake_resp)
        client.get_connection = (lambda *a, **kw: conn)

        resp, body = client.raw_request('GET', '/v1/resources')

        self.assertEqual(resp, fake_resp)
        self.assertIsInstance(body, http.ResponseBodyIterator)


class SessionClientTest(utils.BaseTestCase):

    def test_server_exception_msg_and_traceback(self):
        error_msg = 'another test error'
        error_trace = ("\"Traceback (most recent call last):\\n\\n  "
                       "File \\\"/usr/local/lib/python2.7/...")
        error_body = _get_error_body(error_msg, error_trace)

        fake_session = utils.FakeSession({'Content-Type': 'application/json'},
                                         error_body,
                                         500)

        client = http.SessionClient(session=fake_session)

        error = self.assertRaises(exc.InternalServerError,
                                  client.json_request,
                                  'GET', '/v1/resources')

        self.assertEqual(
            '%(error)s (HTTP 500)\n%(trace)s' % {'error': error_msg,
                                                 'trace': error_trace},
            "%(error)s\n%(details)s" % {'error': str(error),
                                        'details': str(error.details)})

    def test_server_exception_empty_body(self):
        error_body = _get_error_body()

        fake_session = utils.FakeSession({'Content-Type': 'application/json'},
                                         error_body,
                                         500)

        client = http.SessionClient(session=fake_session)

        error = self.assertRaises(exc.InternalServerError,
                                  client.json_request,
                                  'GET', '/v1/resources')

        self.assertEqual('Internal Server Error (HTTP 500)', str(error))

    def test_bypass_url(self):
        fake_response = utils.FakeSessionResponse(
            {}, content="", status_code=201)
        fake_session = mock.MagicMock()
        fake_session.request.side_effect = [fake_response]

        client = http.SessionClient(
            session=fake_session, endpoint_override='http://magnum')

        client.json_request('GET', '/v1/clusters')
        self.assertEqual(
            fake_session.request.call_args[1]['endpoint_override'],
            'http://magnum'
        )

    def test_exception(self):
        fake_response = utils.FakeSessionResponse(
            {}, content="", status_code=504)
        fake_session = mock.MagicMock()
        fake_session.request.side_effect = [fake_response]
        client = http.SessionClient(
            session=fake_session, endpoint_override='http://magnum')
        self.assertRaises(GatewayTimeout,
                          client.json_request,
                          'GET', '/v1/resources')

    def test_construct_http_client_return_httpclient(self):
        client = http._construct_http_client('http://localhost/')

        self.assertIsInstance(client, http.HTTPClient)

    def test_construct_http_client_return_sessionclient(self):
        fake_session = mock.MagicMock()
        client = http._construct_http_client(session=fake_session)

        self.assertIsInstance(client, http.SessionClient)

    def test_raw_request(self):
        fake_response = utils.FakeSessionResponse(
            {'content-type': 'application/octet-stream'},
            content="", status_code=200)
        fake_session = mock.MagicMock()
        fake_session.request.side_effect = [fake_response]

        client = http.SessionClient(
            session=fake_session, endpoint_override='http://magnum')

        resp, resp_body = client.raw_request('GET', '/v1/clusters')

        self.assertEqual(
            fake_session.request.call_args[1]['headers']['Content-Type'],
            'application/octet-stream'
        )
        self.assertEqual(None, resp_body)
        self.assertEqual(fake_response, resp)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from magnumclient.v1 import certificates_shell
from magnumclient.v1 import cluster_templates_shell
from magnumclient.v1 import clusters_shell
from magnumclient.v1 import mservices_shell
from magnumclient.v1 import quotas_shell
from magnumclient.v1 import stats_shell

COMMAND_MODULES = [
    certificates_shell,
    clusters_shell,
    cluster_templates_shell,
    mservices_shell,
    stats_shell,
    quotas_shell,
]



# Copyright 2012 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# W0603: Using the global statement
# W0621: Redefining name %s from outer scope
# pylint: disable=W0603,W0621

import getpass
import inspect
import os
import sys
import textwrap

import decorator
from magnumclient.common.apiclient import exceptions
from oslo_utils import encodeutils
from oslo_utils import strutils
import prettytable

from magnumclient.i18n import _


DEPRECATION_BASE = ('%sThe --%s parameter is deprecated and '
                    'will be removed in a future release. Use the '
                    '<%s> positional parameter %s.')

NAME_DEPRECATION_HELP = DEPRECATION_BASE % ('', 'name', 'name', 'instead')

NAME_DEPRECATION_WARNING = DEPRECATION_BASE % (
    'WARNING: ', 'name', 'name', 'to avoid seeing this message')

CLUSTER_DEPRECATION_HELP = DEPRECATION_BASE % ('', 'cluster', 'cluster',
                                               'instead')

CLUSTER_DEPRECATION_WARNING = DEPRECATION_BASE % (
    'WARNING: ', 'cluster', 'cluster', 'to avoid seeing this message')

MAGNUM_CLIENT_DEPRECATION_WARNING = (
    'WARNING: The magnum client is deprecated and will be removed in a future '
    'release.\nUse the OpenStack client to avoid seeing this message.')


def deprecation_message(preamble, new_name):
    msg = ('%s This parameter is deprecated and will be removed in a future '
           'release. Use --%s instead.' % (preamble, new_name))
    return msg


class MissingArgs(Exception):
    """Supplied arguments are not sufficient for calling a function."""
    def __init__(self, missing):
        self.missing = missing
        msg = _("Missing arguments: %s") % ", ".join(missing)
        super(MissingArgs, self).__init__(msg)


class DuplicateArgs(Exception):
    """More than one of the same argument type was passed."""
    def __init__(self, param, dupes):
        msg = _('Duplicate "%(param)s" arguments: %(dupes)s') % {
            'param': param, 'dupes': ", ".join(dupes)}
        super(DuplicateArgs, self).__init__(msg)


def validate_args(fn, *args, **kwargs):
    """Check that the supplied args are sufficient for calling a function.

    >>> validate_args(lambda a: None)
    Traceback (most recent call last):
        ...
    MissingArgs: Missing argument(s): a
    >>> validate_args(lambda a, b, c, d: None, 0, c=1)
    Traceback (most recent call last):
        ...
    MissingArgs: Missing argument(s): b, d

    :param fn: the function to check
    :param arg: the positional arguments supplied
    :param kwargs: the keyword arguments supplied
    """
    argspec = inspect.getfullargspec(fn)

    num_defaults = len(argspec.defaults or [])
    required_args = argspec.args[:len(argspec.args) - num_defaults]

    def isbound(method):
        return getattr(method, '__self__', None) is not None

    if isbound(fn):
        required_args.pop(0)

    missing = [arg for arg in required_args if arg not in kwargs]
    missing = missing[len(args):]
    if missing:
        raise MissingArgs(missing)


def validate_name_args(positional_name, optional_name):
    if optional_name:
        print(NAME_DEPRECATION_WARNING)
    if positional_name and optional_name:
        raise DuplicateArgs("<name>", (positional_name, optional_name))


def validate_cluster_args(positional_cluster, optional_cluster):
    if optional_cluster:
        print(CLUSTER_DEPRECATION_WARNING)
    if positional_cluster and optional_cluster:
        raise DuplicateArgs("<cluster>", (positional_cluster,
                                          optional_cluster))


def deprecated(message):
    """Decorator for marking a call as deprecated by printing a given message.

    Example:
    >>> @deprecated("Bay functions are deprecated and should be replaced by "
    ...             "calls to cluster")
    ... def bay_create(args):
    ...     pass
    """
    @decorator.decorator
    def wrapper(func, *args, **kwargs):
        print(message)
        return func(*args, **kwargs)
    return wrapper


def deprecation_map(dep_map):
    """Decorator for applying a map of deprecating arguments to a function.

    The map connects deprecating arguments and their replacements. The
    shell.py script uses this map to create mutually exclusive argument groups
    in argparse and also prints a deprecation warning telling the user to
    switch to the updated argument.

    NOTE: This decorator MUST be the outermost in the chain of argument
    decorators to work correctly.

    Example usage:
    >>> @deprecation_map({ "old-argument": "new-argument" })
    ... @args("old-argument", required=True)
    ... @args("new-argument", required=True)
    ... def do_command_line_stuff():
    ...     pass
    """
    def _decorator(func):
        if not hasattr(func, 'arguments'):
            return func

        func.deprecated_groups = []
        for old_param, new_param in dep_map.items():
            old_info, new_info = None, None
            required = False
            for (args, kwargs) in func.arguments:
                if old_param in args:
                    old_info = (args, kwargs)
                    # Old arguments shouldn't be required if they were not
                    # previously, so prioritize old requirement
                    if 'required' in kwargs:
                        required = kwargs['required']
                    # Set to false so argparse doesn't get angry
                    kwargs['required'] = False
                elif new_param in args:
                    new_info = (args, kwargs)
                    kwargs['required'] = False
                if old_info and new_info:
                    break
            # Add a tuple of (old, new, required), which in turn is:
            # ((old_args, old_kwargs), (new_args, new_kwargs), required)
            func.deprecated_groups.append((old_info, new_info, required))
            # Remove arguments that would be duplicated by the groups we made
            func.arguments.remove(old_info)
            func.arguments.remove(new_info)

        return func
    return _decorator


def arg(*args, **kwargs):
    """Decorator for CLI args.

    Example:

    >>> @arg("name", help="Name of the new entity")
    ... def entity_create(args):
    ...     pass
    """
    def _decorator(func):
        add_arg(func, *args, **kwargs)
        return func
    return _decorator


def env(*args, **kwargs):
    """Returns the first environment variable set.

    If all are empty, defaults to '' or keyword arg `default`.
    """
    for arg in args:
        value = os.environ.get(arg)
        if value:
            return value
    return kwargs.get('default', '')


def add_arg(func, *args, **kwargs):
    """Bind CLI arguments to a shell.py `do_foo` function."""

    if not hasattr(func, 'arguments'):
        func.arguments = []

    # NOTE(sirp): avoid dups that can occur when the module is shared across
    # tests.
    if (args, kwargs) not in func.arguments:
        # Because of the semantics of decorator composition if we just append
        # to the options list positional options will appear to be backwards.
        func.arguments.insert(0, (args, kwargs))


def unauthenticated(func):
    """Adds 'unauthenticated' attribute to decorated function.

    Usage:

    >>> @unauthenticated
    ... def mymethod(f):
    ...     pass
    """
    func.unauthenticated = True
    return func


def isunauthenticated(func):
    """Checks if the function does not require authentication.

    Mark such functions with the `@unauthenticated` decorator.

    :returns: bool
    """
    return getattr(func, 'unauthenticated', False)


def print_list(objs, fields, formatters=None, sortby_index=0,
               mixed_case_fields=None, field_labels=None):
    """Print a list or objects as a table, one row per object.

    :param objs: iterable of :class:`Resource`
    :param fields: attributes that correspond to columns, in order
    :param formatters: `dict` of callables for field formatting
    :param sortby_index: index of the field for sorting table rows
    :param mixed_case_fields: fields corresponding to object attributes that
        have mixed case names (e.g., 'serverId')
    :param field_labels: Labels to use in the heading of the table, default to
        fields.
    """
    formatters = formatters or {}
    mixed_case_fields = mixed_case_fields or []
    field_labels = field_labels or fields
    if len(field_labels) != len(fields):
        raise ValueError(_("Field labels list %(labels)s has different number "
                           "of elements than fields list %(fields)s"),
                         {'labels': field_labels, 'fields': fields})

    if sortby_index is None:
        kwargs = {}
    else:
        kwargs = {'sortby': field_labels[sortby_index]}
    pt = prettytable.PrettyTable(field_labels)
    pt.align = 'l'

    for o in objs:
        row = []
        for field in fields:
            data = '-'
            if field in formatters:
                data = formatters[field](o)
            else:
                if field in mixed_case_fields:
                    field_name = field.replace(' ', '_')
                else:
                    field_name = field.lower().replace(' ', '_')
                data = getattr(o, field_name, '')
            if data is None:
                data = '-'
            row.append(data)
        pt.add_row(row)

    print(encodeutils.safe_encode(pt.get_string(**kwargs)).decode())


def keys_and_vals_to_strs(dictionary):
    """Recursively convert a dictionary's keys and values to strings.

    :param dictionary: dictionary whose keys/vals are to be converted to strs
    """
    def to_str(k_or_v):
        if isinstance(k_or_v, dict):
            return keys_and_vals_to_strs(k_or_v)
        elif isinstance(k_or_v, str):
            return str(k_or_v)
        else:
            return k_or_v
    return dict((to_str(k), to_str(v)) for k, v in dictionary.items())


def print_dict(dct, dict_property="Property", wrap=0):
    """Print a `dict` as a table of two columns.

    :param dct: `dict` to print
    :param dict_property: name of the first column
    :param wrap: wrapping for the second column
    """
    pt = prettytable.PrettyTable([dict_property, 'Value'])
    pt.align = 'l'
    for k, v in dct.items():
        # convert dict to str to check length
        if isinstance(v, dict):
            v = str(keys_and_vals_to_strs(v))
        if wrap > 0:
            v = textwrap.fill(str(v), wrap)
        # if value has a newline, add in multiple rows
        # e.g. fault with stacktrace
        if v and isinstance(v, str) and r'\n' in v:
            lines = v.strip().split(r'\n')
            col1 = k
            for line in lines:
                pt.add_row([col1, line])
                col1 = ''
        elif isinstance(v, list):
            val = str([str(i) for i in v])
            if val is None:
                val = '-'
            pt.add_row([k, val])
        else:
            if v is None:
                v = '-'
            pt.add_row([k, v])

    print(encodeutils.safe_encode(pt.get_string()).decode())


def get_password(max_password_prompts=3):
    """Read password from TTY."""
    verify = strutils.bool_from_string(env("OS_VERIFY_PASSWORD"))
    pw = None
    if hasattr(sys.stdin, "isatty") and sys.stdin.isatty():
        # Check for Ctrl-D
        try:
            for __ in range(max_password_prompts):
                pw1 = getpass.getpass("OS Password: ")
                if verify:
                    pw2 = getpass.getpass("Please verify: ")
                else:
                    pw2 = pw1
                if pw1 == pw2 and pw1:
                    pw = pw1
                    break
        except EOFError:
            pass
    return pw


def service_type(stype):
    """Adds 'service_type' attribute to decorated function.

    Usage:

    .. code-block:: python

       @service_type('volume')
       def mymethod(f):
       ...
    """
    def inner(f):
        f.service_type = stype
        return f
    return inner


def get_service_type(f):
    """Retrieves service type from function."""
    return getattr(f, 'service_type', None)


def pretty_choice_list(lst):
    return ', '.join("'%s'" % i for i in lst)


def exit(msg=''):
    if msg:
        print(msg, file=sys.stderr)
    sys.exit(1)


def _format_field_name(attr):
    """Format an object attribute in a human-friendly way."""
    # Split at ':' and leave the extension name as-is.
    parts = attr.rsplit(':', 1)
    name = parts[-1].replace('_', ' ')
    # Don't title() on mixed case
    if name.isupper() or name.islower():
        name = name.title()
    parts[-1] = name
    return ': '.join(parts)


def make_field_formatter(attr, filters=None):
    """Given an object attribute.

    Return a formatted field name and a formatter suitable for passing to
    print_list.
    Optionally pass a dict mapping attribute names to a function. The function
    will be passed the value of the attribute and should return the string to
    display.
    """

    filter_ = None
    if filters:
        filter_ = filters.get(attr)

    def get_field(obj):
        field = getattr(obj, attr, '')
        if field and filter_:
            field = filter_(field)
        return field

    name = _format_field_name(attr)
    formatter = get_field
    return name, formatter


def _get_list_table_columns_and_formatters(fields, objs, exclude_fields=(),
                                           filters=None):
    """Check and add fields to output columns.

    If there is any value in fields that not an attribute of obj,
    CommandError will be raised.
    If fields has duplicate values (case sensitive), we will make them unique
    and ignore duplicate ones.
    :param fields: A list of string contains the fields to be printed.
    :param objs: An list of object which will be used to check if field is
                 valid or not. Note, we don't check fields if obj is None or
                 empty.
    :param exclude_fields: A tuple of string which contains the fields to be
                           excluded.
    :param filters: A dictionary defines how to get value from fields, this
                    is useful when field's value is a complex object such as
                    dictionary.
    :return: columns, formatters.
             columns is a list of string which will be used as table header.
             formatters is a dictionary specifies how to display the value
             of the field.
             They can be [], {}.
    :raise: magnumclient.common.apiclient.exceptions.CommandError.
    """

    if objs and isinstance(objs, list):
        obj = objs[0]
    else:
        obj = None
        fields = None

    columns = []
    formatters = {}

    if fields:
        non_existent_fields = []
        exclude_fields = set(exclude_fields)

        for field in fields.split(','):
            if not hasattr(obj, field):
                non_existent_fields.append(field)
                continue
            if field in exclude_fields:
                continue
            field_title, formatter = make_field_formatter(field, filters)
            columns.append(field_title)
            formatters[field_title] = formatter
            exclude_fields.add(field)

        if non_existent_fields:
            raise exceptions.CommandError(
                _("Non-existent fields are specified: %s") %
                non_existent_fields
            )
    return columns, formatters



# -*- coding: utf-8 -*-
#
# Copyright 2012 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from http import client as http_client
import io
import logging
import os
import socket
import ssl
from urllib import parse as urlparse

from keystoneauth1 import adapter
from oslo_serialization import jsonutils
from oslo_utils import importutils

from magnumclient import exceptions

osprofiler_web = importutils.try_import("osprofiler.web")

LOG = logging.getLogger(__name__)
USER_AGENT = 'python-magnumclient'
CHUNKSIZE = 1024 * 64  # 64kB

API_VERSION = '/v1'
DEFAULT_API_VERSION = 'latest'


def _extract_error_json_text(body_json):
    error_json = {}
    if 'error_message' in body_json:
        raw_msg = body_json['error_message']
        error_json = jsonutils.loads(raw_msg)
    elif 'error' in body_json:
        error_body = body_json['error']
        error_json = {'faultstring': error_body['title'],
                      'debuginfo': error_body['message']}
    else:
        error_body = body_json['errors'][0]
        error_json = {'faultstring': error_body['title']}
        if 'detail' in error_body:
            error_json['debuginfo'] = error_body['detail']
        elif 'description' in error_body:
            error_json['debuginfo'] = error_body['description']
    return error_json


def _extract_error_json(body, resp):
    """Return error_message from the HTTP response body."""
    try:
        content_type = resp.headers.get("Content-Type", "")
    except AttributeError:
        content_type = ""
    if content_type.startswith("application/json"):
        try:
            body_json = resp.json()
            return _extract_error_json_text(body_json)
        except AttributeError:
            body_json = jsonutils.loads(body)
            return _extract_error_json_text(body_json)
        except ValueError:
            return {}
    else:
        try:
            body_json = jsonutils.loads(body)
            return _extract_error_json_text(body_json)
        except ValueError:
            return {}


class HTTPClient(object):

    def __init__(self, endpoint, api_version=DEFAULT_API_VERSION, **kwargs):
        self.endpoint = endpoint
        self.auth_token = kwargs.get('token')
        self.auth_ref = kwargs.get('auth_ref')
        self.api_version = api_version
        self.connection_params = self.get_connection_params(endpoint, **kwargs)

    @staticmethod
    def get_connection_params(endpoint, **kwargs):
        parts = urlparse.urlparse(endpoint)

        # trim API version and trailing slash from endpoint
        path = parts.path
        path = path.rstrip('/').rstrip(API_VERSION)

        _args = (parts.hostname, parts.port, path)
        _kwargs = {'timeout': (float(kwargs.get('timeout'))
                               if kwargs.get('timeout') else 600)}

        if parts.scheme == 'https':
            _class = VerifiedHTTPSConnection
            _kwargs['ca_file'] = kwargs.get('ca_file', None)
            _kwargs['cert_file'] = kwargs.get('cert_file', None)
            _kwargs['key_file'] = kwargs.get('key_file', None)
            _kwargs['insecure'] = kwargs.get('insecure', False)
        elif parts.scheme == 'http':
            _class = http_client.HTTPConnection
        else:
            msg = 'Unsupported scheme: %s' % parts.scheme
            raise exceptions.EndpointException(msg)

        return (_class, _args, _kwargs)

    def get_connection(self):
        _class = self.connection_params[0]
        return _class(*self.connection_params[1][0:2],
                      **self.connection_params[2])

    def log_curl_request(self, method, url, kwargs):
        curl = ['curl -i -X %s' % method]

        for (key, value) in kwargs['headers'].items():
            header = '-H \'%s: %s\'' % (key, value)
            curl.append(header)

        conn_params_fmt = [
            ('key_file', '--key %s'),
            ('cert_file', '--cert %s'),
            ('ca_file', '--cacert %s'),
        ]
        for (key, fmt) in conn_params_fmt:
            value = self.connection_params[2].get(key)
            if value:
                curl.append(fmt % value)

        if self.connection_params[2].get('insecure'):
            curl.append('-k')

        if 'body' in kwargs:
            curl.append('-d \'%s\'' % kwargs['body'])

        curl.append('%s/%s' % (self.endpoint, url.lstrip(API_VERSION)))
        LOG.debug(' '.join(curl))

    @staticmethod
    def log_http_response(resp, body=None):
        status = (resp.version / 10.0, resp.status, resp.reason)
        dump = ['\nHTTP/%.1f %s %s' % status]
        dump.extend(['%s: %s' % (k, v) for k, v in resp.getheaders()])
        dump.append('')
        if body:
            dump.extend([body, ''])
        LOG.debug('\n'.join(dump))

    def _make_connection_url(self, url):
        (_class, _args, _kwargs) = self.connection_params
        base_url = _args[2]
        return '%s/%s' % (base_url, url.lstrip('/'))

    def _http_request(self, url, method, **kwargs):
        """Send an http request with the specified characteristics.

        Wrapper around httplib.HTTP(S)Connection.request to handle tasks such
        as setting headers and error handling.
        """
        # Copy the kwargs so we can reuse the original in case of redirects
        kwargs['headers'] = copy.deepcopy(kwargs.get('headers', {}))
        kwargs['headers'].setdefault('User-Agent', USER_AGENT)
        if self.api_version:
            version_string = 'container-infra %s' % self.api_version
            kwargs['headers'].setdefault(
                'OpenStack-API-Version', version_string)
        if self.auth_token:
            kwargs['headers'].setdefault('X-Auth-Token', self.auth_token)

        self.log_curl_request(method, url, kwargs)
        conn = self.get_connection()

        try:
            conn_url = self._make_connection_url(url)
            conn.request(method, conn_url, **kwargs)
            resp = conn.getresponse()
        except socket.gaierror as e:
            message = ("Error finding address for %(url)s: %(e)s"
                       % dict(url=url, e=e))
            raise exceptions.EndpointNotFound(message)
        except (socket.error, socket.timeout) as e:
            endpoint = self.endpoint
            message = ("Error communicating with %(endpoint)s %(e)s"
                       % dict(endpoint=endpoint, e=e))
            raise exceptions.ConnectionRefused(message)

        body_iter = ResponseBodyIterator(resp)

        # Read body into string if it isn't obviously image data
        body_str = None
        if resp.getheader('content-type', None) != 'application/octet-stream':
            # decoding byte to string is necessary for Python 3.4 compatibility
            # this issues has not been found with Python 3.4 unit tests
            # because the test creates a fake http response of type str
            # the if statement satisfies test (str) and real (bytes) behavior
            body_list = [
                chunk.decode("utf-8") if isinstance(chunk, bytes)
                else chunk for chunk in body_iter
            ]
            body_str = ''.join(body_list)
            self.log_http_response(resp, body_str)
            body_iter = io.StringIO(body_str)
        else:
            self.log_http_response(resp)

        if 400 <= resp.status < 600:
            LOG.warning("Request returned failure status.")
            error_json = _extract_error_json(body_str, resp)
            raise exceptions.from_response(
                resp, error_json.get('faultstring'),
                error_json.get('debuginfo'), method, url)
        elif resp.status in (301, 302, 305):
            # Redirected. Reissue the request to the new location.
            return self._http_request(resp['location'], method, **kwargs)
        elif resp.status == 300:
            raise exceptions.from_response(resp, method=method, url=url)

        return resp, body_iter

    def json_request(self, method, url, **kwargs):
        kwargs.setdefault('headers', {})
        kwargs['headers'].setdefault('Content-Type', 'application/json')
        kwargs['headers'].setdefault('Accept', 'application/json')

        if 'body' in kwargs:
            kwargs['body'] = jsonutils.dumps(kwargs['body'])

        resp, body_iter = self._http_request(url, method, **kwargs)
        content_type = resp.getheader('content-type', None)

        if resp.status == 204 or resp.status == 205 or content_type is None:
            return resp, list()

        if 'application/json' in content_type:
            body = ''.join([chunk for chunk in body_iter])
            try:
                body = jsonutils.loads(body)
            except ValueError:
                LOG.error('Could not decode response body as JSON')
        else:
            body = None

        return resp, body

    def raw_request(self, method, url, **kwargs):
        kwargs.setdefault('headers', {})
        kwargs['headers'].setdefault('Content-Type',
                                     'application/octet-stream')
        return self._http_request(url, method, **kwargs)


class VerifiedHTTPSConnection(http_client.HTTPSConnection):
    """httplib-compatibile connection using client-side SSL authentication

    :see http://code.activestate.com/recipes/
            577548-https-httplib-client-connection-with-certificate-v/
    """

    def __init__(self, host, port, key_file=None, cert_file=None,
                 ca_file=None, timeout=None, insecure=False):
        http_client.HTTPSConnection.__init__(self, host, port)
        self.key_file = key_file
        self.cert_file = cert_file
        if ca_file is not None:
            self.ca_file = ca_file
        else:
            self.ca_file = self.get_system_ca_file()
        self.timeout = timeout
        self.insecure = insecure

    def connect(self):
        """Connect to a host on a given (SSL) port.

        If ca_file is pointing somewhere, use it to check Server Certificate.

        Redefined/copied and extended from httplib.py:1105 (Python 2.6.x).
        This is needed to pass cert_reqs=ssl.CERT_REQUIRED as parameter to
        ssl.wrap_socket(), which forces SSL to check server certificate against
        our client certificate.
        """
        sock = socket.create_connection((self.host, self.port), self.timeout)
        context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)

        if self._tunnel_host:
            self.sock = sock
            self._tunnel()

        if self.insecure is True:
            context.check_hostname = False
            context.verify_mode = ssl.CERT_NONE
        else:
            context.verify_mode = ssl.CERT_REQUIRED
            context.load_verify_locations(self.ca_file)

        if self.cert_file:
            context.load_cert_chain(self.cert_file, self.key_file)

        self.sock = context.wrap_socket(sock, server_hostname=self.host)

    @staticmethod
    def get_system_ca_file():
        """Return path to system default CA file."""
        # Standard CA file locations for Debian/Ubuntu, RedHat/Fedora,
        # Suse, FreeBSD/OpenBSD
        ca_path = ['/etc/ssl/certs/ca-certificates.crt',
                   '/etc/pki/tls/certs/ca-bundle.crt',
                   '/etc/ssl/ca-bundle.pem',
                   '/etc/ssl/cert.pem']
        for ca in ca_path:
            if os.path.exists(ca):
                return ca
        return None


class SessionClient(adapter.LegacyJsonAdapter):
    """HTTP client based on Keystone client session."""

    def __init__(self, user_agent=USER_AGENT, logger=LOG,
                 api_version=DEFAULT_API_VERSION, *args, **kwargs):
        self.user_agent = USER_AGENT
        self.api_version = api_version
        super(SessionClient, self).__init__(*args, **kwargs)

    def _http_request(self, url, method, **kwargs):
        if url.startswith(API_VERSION):
            url = url[len(API_VERSION):]

        kwargs.setdefault('user_agent', self.user_agent)
        kwargs.setdefault('auth', self.auth)
        kwargs.setdefault('endpoint_override', self.endpoint_override)

        # Copy the kwargs so we can reuse the original in case of redirects
        kwargs['headers'] = copy.deepcopy(kwargs.get('headers', {}))
        kwargs['headers'].setdefault('User-Agent', self.user_agent)
        # NOTE(tovin07): osprofiler_web.get_trace_id_headers does not add any
        # headers in case if osprofiler is not initialized.
        if osprofiler_web:
            kwargs['headers'].update(osprofiler_web.get_trace_id_headers())
        if self.api_version:
            version_string = 'container-infra %s' % self.api_version
            kwargs['headers'].setdefault(
                'OpenStack-API-Version', version_string)

        endpoint_filter = kwargs.setdefault('endpoint_filter', {})
        endpoint_filter.setdefault('interface', self.interface)
        endpoint_filter.setdefault('service_type', self.service_type)
        endpoint_filter.setdefault('region_name', self.region_name)

        resp = self.session.request(url, method,
                                    raise_exc=False, **kwargs)

        if 400 <= resp.status_code < 600:
            error_json = _extract_error_json(resp.content, resp)
            raise exceptions.from_response(
                resp, error_json.get('faultstring'),
                error_json.get('debuginfo'), method, url)
        elif resp.status_code in (301, 302, 305):
            # Redirected. Reissue the request to the new location.
            location = resp.headers.get('location')
            resp = self._http_request(location, method, **kwargs)
        elif resp.status_code == 300:
            raise exceptions.from_response(resp, method=method, url=url)
        return resp

    def json_request(self, method, url, **kwargs):
        kwargs.setdefault('headers', {})
        kwargs['headers'].setdefault('Content-Type', 'application/json')
        kwargs['headers'].setdefault('Accept', 'application/json')
        if 'body' in kwargs:
            kwargs['data'] = jsonutils.dumps(kwargs.pop('body'))

        resp = self._http_request(url, method, **kwargs)
        body = resp.content
        content_type = resp.headers.get('content-type', None)
        status = resp.status_code
        if status == 204 or status == 205 or content_type is None:
            return resp, list()
        if 'application/json' in content_type:
            try:
                body = resp.json()
            except ValueError:
                LOG.error('Could not decode response body as JSON')
        else:
            body = None

        return resp, body

    def raw_request(self, method, url, **kwargs):
        kwargs.setdefault('headers', {})
        kwargs['headers'].setdefault('Content-Type',
                                     'application/octet-stream')
        resp = self._http_request(url, method, **kwargs)
        body = resp.content
        status = resp.status_code
        content_type = resp.headers.get('content-type', None)

        if status == 204 or status == 205 or content_type is None:
            return resp, list()
        if 'application/json' in content_type:
            try:
                body = resp.json()
            except ValueError:
                LOG.error('Could not decode response body as JSON')
        else:
            body = None

        return resp, body


class ResponseBodyIterator(object):
    """A class that acts as an iterator over an HTTP response."""

    def __init__(self, resp):
        self.resp = resp

    def __iter__(self):
        while True:
            try:
                yield self.next()
            except StopIteration:
                return

    def __bool__(self):
        return hasattr(self, 'items')

    __nonzero__ = __bool__  # Python 2.x compatibility

    def next(self):
        chunk = self.resp.read(CHUNKSIZE)
        if chunk:
            return chunk
        else:
            raise StopIteration


def _construct_http_client(*args, **kwargs):
    session = kwargs.pop('session', None)
    auth = kwargs.pop('auth', None)

    if session:
        service_type = kwargs.pop('service_type', 'baremetal')
        interface = kwargs.pop('endpoint_type', None)
        region_name = kwargs.pop('region_name', None)
        return SessionClient(session=session,
                             auth=auth,
                             interface=interface,
                             service_type=service_type,
                             region_name=region_name,
                             service_name=None,
                             user_agent='python-magnumclient')
    else:
        return HTTPClient(*args, **kwargs)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from magnumclient.common import cliutils as utils
from magnumclient.common import utils as magnum_utils
from magnumclient import exceptions
from magnumclient.i18n import _


# Maps old parameter names to their new names and whether they are required
# e.g. keypair-id to keypair
DEPRECATING_PARAMS = {
    "--keypair-id": "--keypair",
}


def _show_cluster(cluster):
    del cluster._info['links']
    utils.print_dict(cluster._info)


@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help=_('The last cluster UUID of the previous page; '
                  'displays list of clusters after "marker".'))
@utils.arg('--limit',
           metavar='<limit>',
           type=int,
           help=_('Maximum number of clusters to return.'))
@utils.arg('--sort-key',
           metavar='<sort-key>',
           help=_('Column to sort results by.'))
@utils.arg('--sort-dir',
           metavar='<sort-dir>',
           choices=['desc', 'asc'],
           help=_('Direction to sort. "asc" or "desc".'))
@utils.arg('--fields',
           default=None,
           metavar='<fields>',
           help=_('Comma-separated list of fields to display. '
                  'Available fields: uuid, name, cluster_template_id, '
                  'stack_id, status, master_count, node_count, links, '
                  'create_timeout'
                  )
           )
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_list(cs, args):
    """Print a list of available clusters."""
    clusters = cs.clusters.list(marker=args.marker, limit=args.limit,
                                sort_key=args.sort_key,
                                sort_dir=args.sort_dir)
    columns = [
        'uuid', 'name', 'keypair', 'node_count', 'master_count', 'status'
    ]
    columns += utils._get_list_table_columns_and_formatters(
        args.fields, clusters,
        exclude_fields=(c.lower() for c in columns))[0]

    utils.print_list(clusters, columns,
                     {'versions': magnum_utils.print_list_field('versions')},
                     sortby_index=None)


@utils.deprecation_map(DEPRECATING_PARAMS)
@utils.arg('positional_name',
           metavar='<name>',
           nargs='?',
           default=None,
           help=_('Name of the cluster to create.'))
@utils.arg('--name',
           metavar='<name>',
           default=None,
           help=(_('Name of the cluster to create. %s') %
                 utils.NAME_DEPRECATION_HELP))
@utils.arg('--cluster-template',
           required=True,
           metavar='<cluster_template>',
           help=_('ID or name of the cluster template.'))
@utils.arg('--keypair-id',
           dest='keypair',
           metavar='<keypair>',
           default=None,
           help=utils.deprecation_message(
                'Name of the keypair to use for this cluster.',
                'keypair'))
@utils.arg('--keypair',
           dest='keypair',
           metavar='<keypair>',
           default=None,
           help=_('Name of the keypair to use for this cluster.'))
@utils.arg('--docker-volume-size',
           metavar='<docker-volume-size>',
           type=int,
           help=_('The size in GB for the docker volume to use'))
@utils.arg('--labels', metavar='<KEY1=VALUE1,KEY2=VALUE2;KEY3=VALUE3...>',
           action='append',
           help=_('Arbitrary labels in the form of key=value pairs '
                  'to associate with a cluster. '
                  'May be used multiple times.'))
@utils.arg('--node-count',
           metavar='<node-count>',
           type=int,
           default=1,
           help=_('The cluster node count.'))
@utils.arg('--master-count',
           metavar='<master-count>',
           type=int,
           default=1,
           help=_('The number of master nodes for the cluster.'))
@utils.arg('--discovery-url',
           metavar='<discovery-url>',
           help=_('Specifies custom discovery url for node discovery.'))
@utils.arg('--timeout',
           metavar='<timeout>',
           type=int,
           default=60,
           help=_('The timeout for cluster creation in minutes. The default '
                  'is 60 minutes.'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_create(cs, args):
    """Create a cluster."""
    args.command = 'cluster-create'

    utils.validate_name_args(args.positional_name, args.name)

    cluster_template = cs.cluster_templates.get(args.cluster_template)
    opts = dict()
    opts['name'] = args.positional_name or args.name
    opts['cluster_template_id'] = cluster_template.uuid
    opts['keypair'] = args.keypair
    if args.docker_volume_size is not None:
        opts['docker_volume_size'] = args.docker_volume_size
    if args.labels is not None:
        opts['labels'] = magnum_utils.handle_labels(args.labels)
    opts['node_count'] = args.node_count
    opts['master_count'] = args.master_count
    opts['discovery_url'] = args.discovery_url
    opts['create_timeout'] = args.timeout

    try:
        cluster = cs.clusters.create(**opts)
        # support for non-async in 1.1
        if args.magnum_api_version and args.magnum_api_version == '1.1':
            _show_cluster(cluster)
        else:
            uuid = str(cluster._info['uuid'])
            print("Request to create cluster %s has been accepted." % uuid)
    except Exception as e:
        print("Create for cluster %s failed: %s" %
              (opts['name'], e))


@utils.arg('cluster',
           metavar='<cluster>',
           nargs='+',
           help=_('ID or name of the (cluster)s to delete.'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_delete(cs, args):
    """Delete specified cluster."""
    for id in args.cluster:
        try:
            cs.clusters.delete(id)
            print("Request to delete cluster %s has been accepted." %
                  id)
        except Exception as e:
            print("Delete for cluster %(cluster)s failed: %(e)s" %
                  {'cluster': id, 'e': e})


@utils.arg('cluster',
           metavar='<cluster>',
           help=_('ID or name of the cluster to show.'))
@utils.arg('--long',
           action='store_true', default=False,
           help=_('Display extra associated cluster template info.'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_show(cs, args):
    """Show details about the given cluster."""
    cluster = cs.clusters.get(args.cluster)
    if args.long:
        cluster_template = \
            cs.cluster_templates.get(cluster.cluster_template_id)
        del cluster_template._info['links'], cluster_template._info['uuid']

        for key in cluster_template._info:
            if 'clustertemplate_' + key not in cluster._info:
                cluster._info['clustertemplate_' + key] = \
                    cluster_template._info[key]
    _show_cluster(cluster)


@utils.arg('cluster', metavar='<cluster>', help=_("UUID or name of cluster"))
@utils.arg('--rollback',
           action='store_true', default=False,
           help=_('Rollback cluster on update failure.'))
@utils.arg(
    'op',
    metavar='<op>',
    choices=['add', 'replace', 'remove'],
    help=_("Operations: 'add', 'replace' or 'remove'"))
@utils.arg(
    'attributes',
    metavar='<path=value>',
    nargs='+',
    action='append',
    default=[],
    help=_("Attributes to add/replace or remove "
           "(only PATH is necessary on remove)"))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_update(cs, args):
    """Update information about the given cluster."""
    if args.rollback and args.magnum_api_version and \
            args.magnum_api_version in ('1.0', '1.1', '1.2'):
        raise exceptions.CommandError(
            "Rollback is not supported in API v%s. "
            "Please use API v1.3+." % args.magnum_api_version)
    patch = magnum_utils.args_array_to_patch(args.op, args.attributes[0])
    try:
        cluster = cs.clusters.update(args.cluster, patch, args.rollback)
    except Exception as e:
        print("ERROR: %s" % e.details)
        return

    if args.magnum_api_version and args.magnum_api_version == '1.1':
        _show_cluster(cluster)
    else:
        print("Request to update cluster %s has been accepted." % args.cluster)


@utils.arg('cluster',
           metavar='<cluster>',
           help=_('ID or name of the cluster to retrieve config.'))
@utils.arg('--dir',
           metavar='<dir>',
           default='.',
           help=_('Directory to save the certificate and config files.'))
@utils.arg('--force',
           action='store_true', default=False,
           help=_('Overwrite files if existing.'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_cluster_config(cs, args):
    """Configure native client to access cluster.

    You can source the output of this command to get the native client of the
    corresponding COE configured to access the cluster.

    Example: eval $(magnum cluster-config <cluster-name>).
    """
    args.dir = os.path.abspath(args.dir)
    cluster = cs.clusters.get(args.cluster)
    if (hasattr(cluster, 'api_address') and cluster.api_address is None):
        print("WARNING: The cluster's api_address is not known yet.")
    cluster_template = cs.cluster_templates.get(cluster.cluster_template_id)
    opts = {
        'cluster_uuid': cluster.uuid,
    }

    # Create a new Certificate and Key, sign it
    if not cluster_template.tls_disabled:
        tls = magnum_utils.generate_csr_and_key()
        tls['ca'] = cs.certificates.get(**opts).pem
        opts['csr'] = tls['csr']
        tls['cert'] = cs.certificates.create(**opts).pem
        for k in ('key', 'cert', 'ca'):
            fname = "%s/%s.pem" % (args.dir, k)
            if os.path.exists(fname) and not args.force:
                raise Exception("File %s exists, aborting." % fname)
            else:
                f = open(fname, "w")
                f.write(tls[k])
                f.close()

    print(magnum_utils.config_cluster(cluster, cluster_template,
                                      cfg_dir=args.dir, force=args.force))



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import mock

from magnumclient.osc.v1 import quotas as osc_quotas
from magnumclient.tests.osc.unit.v1 import fakes as magnum_fakes


class TestQuotas(magnum_fakes.TestMagnumClientOSCV1):

    def setUp(self):
        super(TestQuotas, self).setUp()

        self.quotas_mock = self.app.client_manager.container_infra.quotas


class TestQuotasCreate(TestQuotas):

    def setUp(self):
        super(TestQuotasCreate, self).setUp()
        attr = dict()
        attr['name'] = 'fake-quota'
        attr['project_id'] = 'abc'
        attr['resource'] = 'Cluster'
        self._quota = magnum_fakes.FakeQuota.create_one_quota(attr)

        self._default_args = {
            'project_id': 'abc',
            'resource': 'Cluster',
            'hard_limit': 1
        }

        self.quotas_mock.create = mock.Mock()
        self.quotas_mock.create.return_value = self._quota

        self.cmd = osc_quotas.CreateQuotas(self.app, None)

        self.data = tuple(map(lambda x: getattr(self._quota, x),
                              osc_quotas.QUOTA_ATTRIBUTES))

    def test_quotas_create(self):
        arglist = [
            '--project-id', 'abc',
            '--resource', 'Cluster'
        ]
        verifylist = [
            ('project_id', 'abc'),
            ('resource', 'Cluster')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.quotas_mock.create.assert_called_with(**self._default_args)

    def test_quotas_create_with_hardlimit(self):
        arglist = [
            '--project-id', 'abc',
            '--resource', 'Cluster',
            '--hard-limit', '10'
        ]
        verifylist = [
            ('project_id', 'abc'),
            ('resource', 'Cluster'),
            ('hard_limit', 10)
        ]
        self._default_args['hard_limit'] = 10
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.quotas_mock.create.assert_called_with(**self._default_args)

    def test_quotas_create_wrong_projectid(self):
        arglist = ['abcd']
        verifylist = [
            ('project_id', 'abcd')
        ]

        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_quotas_create_missing_args(self):
        arglist = []
        verifylist = []
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_quotas_create_with_wrong_args(self):
        arglist = [
            '--project-id', 'abc',
            '--resources', 'Cluster',  # Misspelling 'resources'
            '--hard-limit', '10'
        ]
        verifylist = []
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)


class TestQuotasDelete(TestQuotas):

    def setUp(self):
        super(TestQuotasDelete, self).setUp()

        self.quotas_mock.delete = mock.Mock()
        self.quotas_mock.delete.return_value = None

        self.cmd = osc_quotas.DeleteQuotas(self.app, None)

    def test_quotas_delete(self):
        arglist = [
            '--project-id', 'abc',
            '--resource', 'Cluster'
        ]
        verifylist = [
            ('project_id', 'abc'),
            ('resource', 'Cluster')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.quotas_mock.delete.assert_called_with('abc', 'Cluster')

    def test_quotas_delete_no_project_id(self):
        arglist = [
            '--resource', 'Cluster'
        ]
        verifylist = [
            ('resource', 'Cluster')
        ]
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_quotas_delete_no_resource(self):
        arglist = [
            '--project-id', 'abc',
        ]
        verifylist = [
            ('project_id', 'abc')
        ]
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_quotas_delete_missing_args(self):
        arglist = []
        verifylist = []
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_quotas_delete_wrong_args(self):
        arglist = [
            '--project-ids', 'abc',  # Misspelling 'ids' instead of 'id'
            '--resource', 'Cluster'
        ]
        verifylist = []
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)


class TestQuotasShow(TestQuotas):

    def setUp(self):
        super(TestQuotasShow, self).setUp()

        attr = dict()
        attr['name'] = 'fake-quota'
        attr['project_id'] = 'abc'
        attr['resource'] = 'Cluster'
        self._quota = magnum_fakes.FakeQuota.create_one_quota(attr)

        self._default_args = {
            'project_id': 'abc',
            'resource': 'Cluster',
        }

        self.quotas_mock.get = mock.Mock()
        self.quotas_mock.get.return_value = self._quota

        self.cmd = osc_quotas.ShowQuotas(self.app, None)

        self.data = tuple(map(lambda x: getattr(self._quota, x),
                              osc_quotas.QUOTA_ATTRIBUTES))

    def test_quotas_show(self):
        arglist = [
            '--project-id', 'abc',
            '--resource', 'Cluster'
        ]
        verifylist = [
            ('project_id', 'abc'),
            ('resource', 'Cluster')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.quotas_mock.get.assert_called_with('abc', 'Cluster')

    def test_quotas_show_missing_args(self):
        arglist = []
        verifylist = []
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)


class TestQuotasUpdate(TestQuotas):

    def setUp(self):
        super(TestQuotasUpdate, self).setUp()
        attr = dict()
        attr['name'] = 'fake-quota'
        attr['project_id'] = 'abc'
        attr['resource'] = 'Cluster'
        self._quota = magnum_fakes.FakeQuota.create_one_quota(attr)

        self._default_args = {
            'project_id': 'abc',
            'resource': 'Cluster',
            'hard_limit': 10
        }

        self.quotas_mock.update = mock.Mock()
        self.quotas_mock.update.return_value = self._quota

        self.cmd = osc_quotas.UpdateQuotas(self.app, None)

    def test_quotas_update(self):
        arglist = [
            '--project-id', 'abc',
            '--resource', 'Cluster',
            '--hard-limit', '10'
        ]
        verifylist = [
            ('project_id', 'abc'),
            ('resource', 'Cluster'),
            ('hard_limit', 10)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.quotas_mock.update.assert_called_with('abc', 'Cluster',
                                                   self._default_args)

    def test_quotas_update_missing_args(self):
        arglist = ['abcd']
        verifylist = [
            ('project_id', 'abcd')
        ]

        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_quotas_update_wrong_args(self):
        arglist = [
            '--project-id', 'abc',
            '--resource', 'Cluster',
            '--hard-limits', '10'  # Misspelling 'hard-limits'
        ]
        verifylist = [
            ('project_id', 'abc'),
            ('resource', 'Cluster'),
            ('hard_limit', 10)
        ]

        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)


class TestQuotasList(TestQuotas):

    def setUp(self):
        super(TestQuotasList, self).setUp()
        attr = dict()
        attr['name'] = 'fake-quota'
        attr['project_id'] = 'abc'
        attr['resource'] = 'Cluster'
        self._quota = magnum_fakes.FakeQuota.create_one_quota(attr)

        self.quotas_mock.list = mock.Mock()
        self.quotas_mock.list.return_value = [self._quota]

        self.cmd = osc_quotas.ListQuotas(self.app, None)

    def test_quotas_list_with_no_options(self):
        arglist = [
        ]
        verifylist = [
            ('limit', None),
            ('sort_key', None),
            ('sort_dir', None),
            ('marker', None),
            ('all_tenants', False)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.quotas_mock.list.assert_called_with(
            limit=None,
            sort_dir=None,
            sort_key=None,
            marker=None,
            all_tenants=False
        )

    def test_quotas_list_wrong_args(self):
        arglist = ['--wrong']
        verifylist = [
            ('wrong')
        ]
        self.assertRaises(magnum_fakes.MagnumParseException,
                          self.check_parser, self.cmd, arglist, verifylist)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnumclient.common import cliutils as utils
from magnumclient.i18n import _


@utils.arg('--project-id',
           required=False,
           metavar='<project-id>',
           help=_('Project ID'))
@utils.deprecated(utils.MAGNUM_CLIENT_DEPRECATION_WARNING)
def do_stats_list(cs, args):
    """Show stats for the given project_id"""
    opts = {
        'project_id': args.project_id
    }

    stats = cs.stats.list(**opts)
    utils.print_dict(stats._info)
